#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Application level testing solution
#
# it should make testing easy as shit

import sys
import time
import urllib
import unittest
import logging
import mimetypes
from Cookie import SimpleCookie
from urllib import quote
from tornado.httpclient import AsyncHTTPClient
from tornado.ioloop import IOLoop
from tornado.stack_context import NullContext
from tornado.escape import json_encode
from tornado import web

from torext.utils import raise_exc_info


COOKIE_HEADER_KEY = 'Set-Cookie'


# The one variable and two functions below are copied from
# requests HTTP library <http://python-requests.org>.
# :copyright: (c) 2013 by Kenneth Reitz.
# :license: Apache 2.0, see LICENSE for more details.

# The unreserved URI characters (RFC 3986)
UNRESERVED_SET = frozenset(
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    + "0123456789-._~")


def unquote_unreserved(uri):
    """Un-escape any percent-escape sequences in a URI that are unreserved
    characters. This leaves all reserved, illegal and non-ASCII bytes encoded.
    """
    parts = uri.split('%')
    for i in range(1, len(parts)):
        h = parts[i][0:2]
        if len(h) == 2 and h.isalnum():
            c = chr(int(h, 16))
            if c in UNRESERVED_SET:
                parts[i] = c + parts[i][2:]
            else:
                parts[i] = '%' + parts[i]
        else:
            parts[i] = '%' + parts[i]
    return ''.join(parts)


def requote_uri(uri):
    """Re-quote the given URI.

    This function passes the given URI through an unquote/quote cycle to
    ensure that it is fully and consistently quoted.
    """
    # Unquote only the unreserved characters
    # Then quote only illegal characters (do not quote reserved, unreserved,
    # or '%')
    return quote(unquote_unreserved(uri), safe="!#$%&'()*+,/:;=?@[]~")


class TestClient(object):
    def __init__(self, app, raise_handler_exc=False):
        # TODO add `host` kwarg
        if app.is_running:
            raise RuntimeError('You should not instance TestClient\
                               when applicaion is running')

        self.raise_handler_exc = raise_handler_exc

        # start::tornado.testing.AsyncTestCase
        self.__stopped = False
        self.__running = False
        self.__failure = None
        self.__stop_args = None
        self.__timeout = None
        # end::tornado.testing.AsyncTestCase

        self._handler_exc_info = None

        app.update_settings({
            'TESTING': True,
            'PROCESSES': 1
        })
        # redo setup to make settings changes effective
        app.setup()

        self.app = app
        self.setup()

    def setup(self):
        """Setup facilities for running the server

        NOTE if you get all "Address already in use" error except the first one,
        the bug may well happened in this function.
        """
        # start::tornado.testing.AsyncTestCase
        #self.io_loop = self.get_new_ioloop()
        # end::tornado.testing.AsyncTestCase
        #self.app.io_loop = self.io_loop

        # init app.io_loop, app.application and app.http_server
        self.app._init_infrastructures()
        self.io_loop = self.app.io_loop

        if self.raise_handler_exc:
            self.patch_app_handlers()

        self.http_server = self.app.http_server
        self.http_client = AsyncHTTPClient(io_loop=self.io_loop)

    def close(self):
        """CLose http_server, io_loop by sequence, to ensure the environment
        is cleaned up and invoking `setup` successfully within next test function

        It is suggested to be called in `TestCase.tearDown`
        """
        # start::tornado.testing.AsyncHTTPTestCase
        self.http_server.stop()
        if (not IOLoop.initialized() or
                self.http_client.io_loop is not IOLoop.instance()):
            self.http_client.close()
        # end::tornado.testing.AsyncHTTPTestCase

        # start::tornado.testing.AsyncTestCase
        if (not IOLoop.initialized() or
                self.io_loop is not IOLoop.instance()):
            # Try to clean up any file descriptors left open in the ioloop.
            # This avoids leaks, especially when tests are run repeatedly
            # in the same process with autoreload (because curl does not
            # set FD_CLOEXEC on its file descriptors)
            self.io_loop.close(all_fds=True)
        # end::tornado.testing.AsyncTestCase

    def _parse_cookies(self, resp):
        if COOKIE_HEADER_KEY in resp.headers:
            c = SimpleCookie(resp.headers.get(COOKIE_HEADER_KEY))
        elif COOKIE_HEADER_KEY.lower() in resp.headers:
            c = SimpleCookie(resp.headers.get(COOKIE_HEADER_KEY.lower()))
        else:
            c = None
        resp.cookies = c

    def _add_cookies(self, cookies, kwgs):
        # SimpleCookie is inherited from dict, so judge it first
        if isinstance(cookies, SimpleCookie):
            c = cookies
        elif isinstance(cookies, dict):
            c = SimpleCookie()
            for k, v in cookies.iteritems():
                c[k] = v
        else:
            raise TypeError('cookies kwarg should be dict or SimpleCookie instance')

        if 'headers' in kwgs:
            headers = kwgs['headers']
        else:
            headers = kwgs['headers'] = {}

        headers['Cookie'] = c.output().lstrip(COOKIE_HEADER_KEY + ': ')
        #print 'headers', headers

    def stop(self, _arg=None, **kwargs):
        '''Stops the ioloop, causing one pending (or future) call to wait()
        to return.

        Keyword arguments or a single positional argument passed to stop() are
        saved and will be returned by wait().
        '''
        assert _arg is None or not kwargs
        self.__stop_args = kwargs or _arg
        if self.__running:
            self.io_loop.stop()
            self.__running = False
        self.__stopped = True

    def wait(self, condition=None, timeout=5):
        """Runs the IOLoop until stop is called or timeout has passed.

        In the event of a timeout, an exception will be thrown.

        If condition is not None, the IOLoop will be restarted after stop()
        until condition() returns true.
        """
        if not self.__stopped:
            if timeout:
                def timeout_func():
                    try:
                        raise self.failureException(
                            'Async operation timed out after %s seconds' %
                            timeout)
                    except Exception:
                        self.__failure = sys.exc_info()
                    self.stop()
                if self.__timeout is not None:
                    self.io_loop.remove_timeout(self.__timeout)
                self.__timeout = self.io_loop.add_timeout(time.time() + timeout, timeout_func)
            while True:
                self.__running = True
                with NullContext():
                    # Wipe out the StackContext that was established in
                    # self.run() so that all callbacks executed inside the
                    # IOLoop will re-run it.
                    self.io_loop.start()
                if (self.__failure is not None or
                        condition is None or condition()):
                    break
        assert self.__stopped
        self.__stopped = False
        self.__rethrow()
        result = self.__stop_args
        self.__stop_args = None
        return result

    def __rethrow(self):
        if self.__failure is not None:
            failure = self.__failure
            self.__failure = None
            raise_exc_info(failure)

    def request(self, method, path,
                data=None, json=False,
                files=None,
                cookies=None, **kwgs):

        kwgs['method'] = method

        # `path` should be utf-8 encoded string to complete requote process
        if isinstance(path, unicode):
            path = path.encode('utf8')
        path = requote_uri(path)

        # `body` must be passed if method is one of those three
        if method in ['POST', 'PUT', 'PATCH']:
            headers = kwgs.setdefault('headers', {})
            body = ''
            if files:
                boundary = '1234567890'
                headers['Content-Type'] = 'multipart/form-data; boundary=%s' % boundary
                L = []
                if data:
                    for k, v in data.iteritems():
                        L.append('--' + boundary)
                        L.append('Content-Disposition: form-data; name="%s"' % k)
                        L.append('')
                        L.append(v)
                for k, f in files.iteritems():
                    L.append('--' + boundary)
                    L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (k, f[0]))
                    L.append('Content-Type: %s' % mimetypes.guess_type(f[0])[0] or 'application/octet-stream')
                    L.append('')
                    L.append(f[1])
                L.append('--%s--' % boundary)
                L.append('')
                body = '\r\n'.join(L)
            else:
                if data:
                    if json:
                        body = json_encode(data)
                        headers['Content-Type'] = 'application/json'
                    else:
                        headers['Content-Type'] = 'application/x-www-form-urlencoded'
                        body = urllib.urlencode(data)
            kwgs['body'] = body
        else:
            if data:
                path = '%s?%s' % (path, urllib.urlencode(data))

        if cookies:
            self._add_cookies(cookies, kwgs)

        #print 'fetch kwgs', kwgs
        url = self.get_url(path)
        logging.debug('testing fetch url: %s', url)
        self.http_client.fetch(url, self.stop, **kwgs)
        resp = self.wait()

        self._parse_cookies(resp)

        return resp

    def get(self, *args, **kwgs):
        return self.request('GET', *args, **kwgs)

    def delete(self, *args, **kwgs):
        return self.request('DELETE', *args, **kwgs)

    def post(self, *args, **kwgs):
        return self.request('POST', *args, **kwgs)

    def put(self, *args, **kwgs):
        return self.request('PUT', *args, **kwgs)

    def patch(self, *args, **kwgs):
        return self.request('PATCH', *args, **kwgs)

    def get_protocol(self):
        return 'http'

    def get_http_port(self):
        return self.app.settings['PORT']

    def get_url(self, path):
        """Returns an absolute url for the given path on the test server."""
        return '%s://localhost:%s%s' % (self.get_protocol(),
                                        self.get_http_port(), path)

    def get_handler_exc(self):
        if self._handler_exc_info:
            raise_exc_info(self._handler_exc_info)

    def patch_handler(self, hdr):
        if not isinstance(hdr, web.StaticFileHandler):
            hdr._testing_app_client = self
            hdr._handle_request_exception = _handle_request_exception

    def patch_app_handlers(self):
        for host_pattern, rules in self.app.application.handlers:
            for r in rules:
                self.patch_handler(r.handler_class)


def _handle_request_exception(self, e):
    import httplib
    import logging
    from tornado.web import HTTPError

    if isinstance(e, HTTPError):
        if e.log_message:
            format = "%d %s: " + e.log_message
            args = [e.status_code, self._request_summary()] + list(e.args)
            logging.warning(format, *args)
        if e.status_code not in httplib.responses:
            logging.error("Bad HTTP status code: %d", e.status_code)
            self.send_error(500, exc_info=sys.exc_info())
        else:
            self.send_error(e.status_code, exc_info=sys.exc_info())
    else:
        self._testing_app_client._handler_exc_info = sys.exc_info()

        logging.error("Uncaught exception %s\n%r", self._request_summary(),
                      self.request, exc_info=True)
        self.send_error(500, exc_info=sys.exc_info())


class AppTestCase(unittest.TestCase):
    def setUp(self):
        self.c = self.get_client()

    def tearDown(self):
        self.c.close()

    def get_client(self):
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from torext import settings
from torext.errors import URLRouteError
from torext.log import app_log


class ModuleSearcher(object):
    def __init__(self, label):
        assert settings['PROJECT'], 'you must set PROJECT first'
        self.import_path = settings['PROJECT'] + '.' + label
        self._handlers = []

    def get_handlers(self):
        module = __import__(self.import_path, fromlist=[settings['PROJECT']])

        try:
            self._handlers = getattr(module, 'handlers')
        except AttributeError, e:
            # TODO enhanced traceback
            raise URLRouteError('Caught error when router was getting handlers from module: %s' % e)

        app_log.debug('got handlers from module %s' % self.import_path)

        for i in self._handlers:
            if isinstance(i[1], ModuleSearcher):
                raise URLRouteError('You should not use `include` in subapp handlers')

        return self._handlers


class Router(object):
    def __init__(self, specs, prefix=None):
        self.specs = specs
        self.prefix = prefix
        self._handlers = []

    def get_handlers(self):
        for spec in self.specs:
            if isinstance(spec[1], str):
                searcher = ModuleSearcher(spec[1])
                for searcher_spec in searcher.get_handlers():
                    _searcher_spec = list(searcher_spec)
                    _searcher_spec[0] = spec[0] + _searcher_spec[0]
                    self.add(tuple(_searcher_spec))
            else:
                self.add(spec)

        return self._handlers

    def add(self, spec):
        if self.prefix:
            spec = (self.prefix + spec[0], spec[1])
        app_log.debug('add url spec in router: %s' % str(spec))
        self._handlers.append(spec)


def include(label):
    return ModuleSearcher(label)


def format_pattern(ptn):
    if ptn.endswith('/') and len(ptn) > 1:
        ptn = ptn[:-1]
    return ptn



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# make a project sketch.

import os
import re
import sys
import shutil
import optparse
import torext
from torext.utils import import_module


parser = optparse.OptionParser(usage='Usage: %prog project')


class CommandInputError(Exception):
    def __init__(self, msg):
        self.msg = msg
        print self.__str__() + '\n'
        parser.print_help()
        sys.exit()

    def __str__(self):
        return self.__class__.__name__ + ': %s' % self.msg


def main():
    options, args = parser.parse_args()

    try:
        name = args[0]
    except IndexError:
        raise CommandInputError('one arg is required')
    # If it's not a valid directory/file name.
    if not re.search(r'^[_a-zA-Z]\w*$', name):
        # Provide a smart error message, depending on the error.
        if not re.search(r'^[_a-zA-Z]', name):
            message = 'make sure the name begins with a letter or underscore'
        else:
            message = 'use only numbers, letters and underscores'
        raise CommandInputError(message)

    cwd = os.getcwd()
    torext_path = os.path.dirname(torext.__file__)

    # Check that the project name cannot be imported.
    try:
        import_module(name)
    except ImportError:
        pass
    else:
        raise CommandInputError("%r conflicts with the name of an existing Python module\
            and cannot be used as a project name. Please try another name." % name)

    template_path = os.path.join(torext_path, 'templates/project')
    target_path = os.path.join(cwd, name)
    assert not os.path.exists(target_path), 'You indicate an existing dir,\
        could not be used for sketch'

    shutil.copytree(template_path, target_path)
    with open(os.path.join(target_path, 'settings.py'), 'r') as f:
        settings_str = f.read()
        settings_str = settings_str.replace('{project_name}', name)
    with open(os.path.join(target_path, 'settings.py'), 'w') as f:
        f.write(settings_str)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# from shiao's tornado.third.renren
# sina weibo
# oauth2 mixin classes

import logging
import urllib
import urlparse
import hashlib
import requests
import time
from tornado import escape
from tornado import httpclient
from tornado import gen
from tornado.escape import json_decode
from tornado.httputil import url_concat
from tornado.auth import OAuthMixin, OAuth2Mixin
from tornado.util import bytes_type

from torext import settings

#
# TODO move the consumer_* arguments out of classes
#

####################
# oauth 1.0 & 1.0a #
####################


class TwitterOAuthMixin(OAuthMixin):
    # copy from tornado.auth
    _OAUTH_REQUEST_TOKEN_URL = "http://api.twitter.com/oauth/request_token"
    _OAUTH_ACCESS_TOKEN_URL = "http://api.twitter.com/oauth/access_token"
    _OAUTH_AUTHORIZE_URL = "http://api.twitter.com/oauth/authorize"
    _OAUTH_AUTHENTICATE_URL = "http://api.twitter.com/oauth/authenticate"

    def authenticate_redirect(self):
        http = httpclient.AsyncHTTPClient()
        http.fetch(self._oauth_request_token_url(), self.async_callback(
            self._on_request_token, self._OAUTH_AUTHENTICATE_URL, None))

    def twitter_request(self, path, callback, access_token=None,
                        post_args=None, **args):
        # Add the OAuth resource request signature if we have credentials
        # NOTE varibles::
        # :url        used to send request, and bear encoded `args`.
        # :args       keyword-arguments that additionaly added to oauth parameters,
        #             lay on `url`.
        # :post_args  use to judge request method, must be passed as post-data
        # :all_args   as every argument in request take activity
        #             when oauth-parameters is generated, `all_args` contain `args` and `post_args`
        url = "http://api.twitter.com/1" + path + ".json"
        if access_token:
            all_args = {}
            all_args.update(args)
            all_args.update(post_args or {})
            method = "POST" if post_args is not None else "GET"
            oauth = self._oauth_request_parameters(
                url, access_token, all_args, method=method)
            args.update(oauth)
        if args:
            url += "?" + urllib.urlencode(args)
        callback = self.async_callback(self._on_twitter_request, callback)
        http = httpclient.AsyncHTTPClient()
        if post_args is not None:
            http.fetch(url, method="POST", body=urllib.urlencode(post_args),
                       callback=callback)
        else:
            http.fetch(url, callback=callback)

    def _on_twitter_request(self, callback, response):
        if response.error:
            logging.warning("Error response %s fetching %s", response.error,
                            response.request.url)
            callback(None)
            return
        callback(escape.json_decode(response.body))

    def _oauth_consumer_token(self):
        return dict(
            key=settings['TWITTER']['consumer_key'],
            secret=settings['TWITTER']['consumer_secret'])

    def _oauth_get_user(self, access_token, callback):
        callback = self.async_callback(self._parse_user_response, callback)
        self.twitter_request(
            "/users/show/" + access_token["screen_name"],
            access_token=access_token, callback=callback)

    def _parse_user_response(self, callback, user):
        if user:
            user["username"] = user["screen_name"]
        callback(user)


class WeiboOAuthMixin(OAuthMixin):
    _OAUTH_REQUEST_TOKEN_URL = "http://api.t.sina.com.cn/oauth/request_token"
    _OAUTH_ACCESS_TOKEN_URL = "http://api.t.sina.com.cn/oauth/access_token"
    _OAUTH_AUTHORIZE_URL = "http://api.t.sina.com.cn/oauth/authorize"
    _OAUTH_API_DOMAIN = "api.t.sina.com.cn"

    def weibo_request(self, path, callback, access_token=None,
                      post_args=None, **args):
        url = "http://" + self._OAUTH_API_DOMAIN + path + ".json"
        if access_token:
            all_args = {}
            all_args.update(args)
            all_args.update(post_args or {})
            method = "POST" if post_args is not None else "GET"
            oauth = self._oauth_request_parameters(
                url, access_token, all_args, method=method)
            args.update(oauth)
        if args:
            url += "?" + urllib.urlencode(args)
        callback = self.async_callback(self._on_weibo_request, callback)
        http = httpclient.AsyncHTTPClient()
        if post_args is not None:
            http.fetch(url, method="POST", body=urllib.urlencode(post_args),
                       callback=callback)
        else:
            http.fetch(url, callback=callback)

    def _on_weibo_request(self, callback, response):
        if response.error:
            logging.warning("Error response %s fetching %s", response.error,
                            response.request.url)
            callback(None)
            return
        callback(escape.json_decode(response.body))

    def _oauth_consumer_token(self):
        return dict(key=settings.networks['weibo']['consumer_key'],
                    secret=settings.networks['weibo']['consumer_secret'])

    def _oauth_get_user(self, access_token, callback):
        callback = self.async_callback(self._parse_user_response, callback)
        self.weibo_request(
            "/users/show",
            access_token=access_token, callback=callback,
            user_id=access_token["user_id"])

    def _parse_user_response(self, callback, user):
        if user:
            user["username"] = user["screen_name"]
        callback(user)


class DoubanOAuthMixin(OAuthMixin):
    _OAUTH_REQUEST_TOKEN_URL = "http://www.douban.com/service/auth/request_token"
    _OAUTH_ACCESS_TOKEN_URL = "http://www.douban.com/service/auth/access_token"
    _OAUTH_AUTHORIZE_URL = "http://www.douban.com/service/auth/authorize"
    _OAUTH_API_DOMAIN = "api.douban.com"
    _OAUTH_VERSION = "1.0"

    def douban_request(self, path, callback, access_token=None,
                       post_args=None, **args):
        # due to some special string like ``@`` may appear in url,
        # and they are required to be quoted before generated to be oauth parameters,
        # (unfortunately tornado don't voluntarily do that)
        # we forwardly quote the url before it is handled.
        url = urllib.quote("http://" + self._OAUTH_API_DOMAIN + path, ':/')

        # reset `format` value in args
        args['alt'] = 'json'

        if access_token:
            all_args = {}
            all_args.update(args)
            all_args.update(post_args or {})
            method = "POST" if post_args is not None else "GET"
            oauth = self._oauth_request_parameters(
                url, access_token, all_args, method=method)
            args.update(oauth)
        http = httpclient.AsyncHTTPClient()
        callback = self.async_callback(self._on_douban_request, callback)

        if post_args is not None:
            # douban says that they can't deal request properly
            # when oauth parameters passing in post data,
            # instead, passing it in HTTP Headers
            # NOTE that `url` here is different from `url` in below fetch function:
            # it is plain, doesn't contain encoded args.
            http.fetch(url, method="POST",
                       headers=args,
                       body=post_args,
                       callback=callback)
        else:
            if args:
                url += "?" + urllib.urlencode(args)
            http.fetch(url, callback=callback)

    def _on_douban_request(self, callback, response):
        if response.error:
            logging.warning("Error response %s fetching %s", response.error,
                            response.request.url)
            callback(None)
            return
        callback(escape.json_decode(response.body))

    def _oauth_consumer_token(self):
        return dict(key=settings.networks['douban']['consumer_key'],
                    secret=settings.networks['douban']['consumer_secret'])

    def _oauth_get_user(self, access_token, callback):
        callback = self.async_callback(self._parse_user_response, callback)
        self.douban_request(
            "/people/@me",
            access_token=access_token, callback=callback)

    def _parse_user_response(self, callback, user):
        if user:
            user["uid"] = user["db:uid"]["$t"]
        callback(user)


class TencentOAuthMixin(OAuthMixin):
    _OAUTH_REQUEST_TOKEN_URL = "http://open.t.qq.com/cgi-bin/request_token"
    _OAUTH_ACCESS_TOKEN_URL = "http://open.t.qq.com/cgi-bin/access_token"
    _OAUTH_AUTHORIZE_URL = "http://open.t.qq.com/cgi-bin/authorize"
    _OAUTH_API_DOMAIN = "open.t.qq.com/api"
    #_OAUTH_VERSION = "1.0"

    def tencent_request(self, path, callback, access_token=None,
                        post_args=None, **args):
        # as Tencent is so fucking shit
        # that it use OAuth1.0a but only let the parameter
        # ``oauth_version`` pass value of 1.0,
        # there will be very fucking weird problem occurs
        # if we don't manually change the _OAUTH_VERSION to be 1.0
        # before oauth parameters are generated
        self._OAUTH_VERSION = '1.0'

        url = urllib.quote("http://" + self._OAUTH_API_DOMAIN + path, ':/')

        # reset `format` value in args
        args['format'] = 'json'

        if access_token:
            all_args = {}
            all_args.update(args)
            all_args.update(post_args or {})
            method = "POST" if post_args is not None else "GET"
            oauth = self._oauth_request_parameters(
                url, access_token, all_args, method=method)
            args.update(oauth)
        if args:
            url += "?" + urllib.urlencode(args)
        http = httpclient.AsyncHTTPClient()
        callback = self.async_callback(self._on_tencent_request, callback)

        if post_args is not None:
            http.fetch(url, method="POST", body=urllib.urlencode(post_args),
                       callback=callback)
        else:
            http.fetch(url, callback=callback)

    def _on_tencent_request(self, callback, response):
        if response.error:
            logging.warning("Error response %s fetching %s", response.error,
                            response.request.url)
            callback(None)
            return
        callback(escape.json_decode(response.body))

    def _oauth_consumer_token(self):
        return dict(key=settings.networks['tencent']['consumer_key'],
                    secret=settings.networks['tencent']['consumer_secret'])

    def _oauth_get_user(self, access_token, callback):
        callback = self.async_callback(self._parse_user_response, callback)
        self.tencent_request(
            "/user/info",
            access_token=access_token, callback=callback)

    def _parse_user_response(self, callback, user):
        print 'user', user
        if user:
            user["username"] = user["data"]["name"]
        callback(user)


#############
# oauth 2.0 #
#############


class FacebookOAuth2Mixin(OAuth2Mixin):
    # copy from tornado.auth
    """Facebook authentication using the new Graph API and OAuth2."""
    _OAUTH_ACCESS_TOKEN_URL = "https://graph.facebook.com/oauth/access_token?"
    _OAUTH_AUTHORIZE_URL = "https://graph.facebook.com/oauth/authorize?"

    def authorize_redirect(self):
        options = settings['FACEBOOK']
        extra_params = None
        if options.get('scope', None):
            extra_params = {'scope': options['scope']}
        super(FacebookOAuth2Mixin, self).authorize_redirect(
            redirect_uri=options['redirect_uri'],
            client_id=options['consumer_key'],
            extra_params=extra_params)

    def get_authenticated_user(self, code, callback, extra_fields=None):
        http = httpclient.AsyncHTTPClient()
        options = settings['FACEBOOK']
        args = {
            "code": code,
            "client_id": options['consumer_key'],
            "client_secret": options['consumer_secret'],
            "redirect_uri": options['redirect_uri'],
        }

        fields = set(['id', 'name', 'first_name', 'last_name',
                      'locale', 'picture', 'link'])
        if extra_fields:
            fields.update(extra_fields)

        http.fetch(self._oauth_request_token_url(**args),
                   self.async_callback(self._on_access_token, callback, fields))

    def _on_access_token(self, callback, fields, response):
        if response.error:
            logging.warning('Facebook auth error: %s' % response)
            callback(None)
            return

        args = escape.parse_qs_bytes(escape.native_str(response.body))
        session = {
            "access_token": args["access_token"][-1],
            "expires": args.get("expires")
        }

        self.facebook_request(
            path="/me",
            callback=self.async_callback(
                self._on_get_user_info, callback, session, fields),
            access_token=session["access_token"],
            fields=",".join(fields)
        )

    def _on_get_user_info(self, callback, session, fields, user):
        if user is None:
            callback(None)
            return

        fieldmap = {}
        for field in fields:
            fieldmap[field] = user.get(field)

        fieldmap.update({"access_token": session["access_token"], "session_expires": session.get("expires")})
        callback(fieldmap)

    def facebook_request(self, path, callback, access_token=None,
                         post_args=None, **args):
        url = "https://graph.facebook.com" + path
        all_args = {}
        if access_token:
            all_args["access_token"] = access_token
            all_args.update(args)
            all_args.update(post_args or {})
        if all_args:
            url += "?" + urllib.urlencode(all_args)
        callback = self.async_callback(self._on_facebook_request, callback)
        http = httpclient.AsyncHTTPClient()
        if post_args is not None:
            http.fetch(url, method="POST", body=urllib.urlencode(post_args),
                       callback=callback)
        else:
            http.fetch(url, callback=callback)

    def _on_facebook_request(self, callback, response):
        if response.error:
            logging.warning("Error response %s fetching %s", response.error,
                            response.request.url)
            callback(None)
            return
        callback(escape.json_decode(response.body))


class WeiboOAuth2Mixin(object):
    _OAUTH_AUTHORIZE_URL = "https://api.weibo.com/oauth2/authorize"
    _OAUTH_ACCESS_TOKEN_URL = "https://api.weibo.com/oauth2/access_token"

    _OAUTH_URL = "https://api.weibo.com/2/"

    @gen.engine
    def weibo_request(self, path, access_token, callback, data=None, **args):
        url = self._OAUTH_URL + path
        if data is not None:
            requests.post(url, data=data)
        else:
            pass
        return

    @gen.engine
    def get_authenticated_user(self, code, callback):
        self.get_access_token(
            code, callback=(yield gen.Callback('WeiboGraphMixin.get_authenticated_user')))

        token_dict = yield gen.Wait('WeiboGraphMixin.get_authenticated_user')
        if not token_dict:
            callback(None)
            return

        callback('success')

    def authorize_redirect(self,
                           response_type='code',
                           extra_display=None,
                           state=None,
                           **kwgs):
        consumer_token = self._oauth_consumer_token()
        assert (response_type in ('code', 'token')), 'argument:response_type incorrect'
        display_list = ['default', ]
        if extra_display is not None:
            display_list.extend(extra_display)
        display = ','.join(display_list)
        all_args = {
            'client_id': consumer_token['client_id'],
            'redirect_uri': settings.networks['weibo']['redirect_uri'],
            'response_type': response_type,
            'display': display,
        }
        all_args.update(kwgs)
        self.redirect(url_concat(self._OAUTH_AUTHORIZE_URL, all_args))

    @gen.engine
    def get_access_token(self, code, callback,
                         grant_type='authorization_code',
                         **kwgs):
        consumer_token = self._oauth_consumer_token()
        assert (grant_type in ('authorization_code', 'password', 'refresh_token')), 'argument:response_type incorrect'
        all_args = {
            'client_id': consumer_token['client_id'],
            'client_secret': consumer_token['client_secret'],
            'grant_type': grant_type,
        }
        if 'authorization_code' == grant_type:
            all_args['code'] = code
            all_args['redirect_uri'] = settings.networks['weibo']['redirect_uri']
        elif 'password' == grant_type:
            for k in ('username', 'password'):
                assert k in kwgs, 'argument:%s is required' % k
                all_args[k] = kwgs[k]
        else:  # 'refresh_token' == grant_type
            for k in ('refresh_token', ):
                assert k in kwgs, 'argument:%s is required' % k
                all_args[k] = kwgs[k]

        # NOTE will cause this error:
        # SSL routines:SSL3_GET_SERVER_CERTIFICATE:certificate verify failed
        #
        #http = httpclient.AsyncHTTPClient()
        #req = httpclient.HTTPRequest(self._OAUTH_ACCESS_TOKEN_URL,
                #body=urllib.urlencode(all_args))
        #http.fetch(req,
                   #callback=(yield gen.Callback('WeiboGraphMixin.get_access_token')))
        #response = yield gen.Wait('WeiboGraphMixin.get_access_token')
        response = requests.post(self._OAUTH_ACCESS_TOKEN_URL, all_args)

        if response.error:
            logging.warning("Error response %s fetching %s",
                            response.error, response.url)
            callback(None)
            return

        token_dict = json_decode(response.read())
        callback(token_dict)

    def _oauth_consumer_token(self):
        token = dict(client_id=settings.networks['weibo']['consumer_key'],
                     client_secret=settings.networks['weibo']['consumer_secret'],)
        return token


class RenrenOAuth2Mixin(object):
    _OAUTH_AUTHORIZE_URL = "https://graph.renren.com/oauth/authorize"
    _OAUTH_ACCESS_TOKEN_URL = "https://graph.renren.com/oauth/token"

    _OAUTH_URL = "https://graph.renren.com/"

    @gen.engine
    def renren_request(self, path, access_token, callback,
                       post_args=None, **args):
        http = httpclient.AsyncHTTPClient()
        url = self._OAUTH_URL + path
        if post_args is not None:
            #TODO as renren has not release graph resource support
            # method POST
            pass
        else:
            args['oauth_token'] = access_token
            http.fetch(
                url_concat(url, args),
                callback=(yield gen.Callback('_RenrenGraphMixin.renren_request'))
            )
        response = yield gen.Wait('_RenrenGraphMixin.renren_request')
        if response.error and not response.body:
            logging.warning("Error response %s fetching %s",
                            response.error, response.request.url)
            callback(None)
            return
        callback(response)
        return

    @gen.engine
    def get_authenticated_user(self, redirect_uri, callback, scope=None, **args):
        """
        class RenrenHandler(tornado.web.RequestHandler, RenrenGraphMixin):
            @tornado.web.asynchronous
            @gen.engine
            def get(self):
                self.get_authenticated_user(
                    callback=(yield gen.Callback('key')),
                    redirect_uri=url)
                user = yield gen.Wait('key')
                if not user:
                    raise web.HTTPError(500, "Renren auth failed")
                # do something else
                self.finish()
        """

        code = self.get_argument('code', None)
        if not code:
            self.authorize_redirect(redirect_uri, scope=scope, **args)
            return
        self.get_access_token(
            code, callback=(yield gen.Callback('_RenrenGraphMixin.get_authenticated_user')),
            redirect_uri=redirect_uri)

        response = yield gen.Wait('_RenrenGraphMixin.get_authenticated_user')
        if not response:
            callback(None)
            return
        try:
            user = json_decode(response.body)
        except:
            logging.warning("Error response %s fetching %s",
                            response.body, response.request.url)
            callback(None)
            return
        if 'error' in user:
            logging.warning("Error response %s fetching %s",
                            user['error_description'], response.request.url)
            callback(None)
            return

        #{{{ get session key
        self.renren_request('renren_api/session_key', user['access_token'],
                            callback=(yield gen.Callback('_RenrenGraphMixin._session_key')))
        response = yield gen.Wait('_RenrenGraphMixin._session_key')
        if response.error and not response.body:
            logging.warning("Error response %s fetching %s",
                            response.error, response.request.url)
        elif response.error:
            logging.warning("Error response %s fetching %s: %s",
                            response.error, response.request.url, response.body)
        else:
            try:
                user['session'] = json_decode(response.body)
            except:
                pass
        #}}} #TODO delete when renren graph api released
        callback(user)
        return

    def authorize_redirect(self, redirect_uri, response_type='code', scope=None, **args):
        consumer_token = self._oauth_consumer_token()
        all_args = {
            'client_id': consumer_token['key'],
            'redirect_uri': redirect_uri,
            'response_type': response_type,
        }
        if scope:
            all_args.update({'scope': scope})
        args.update(all_args)
        self.redirect(url_concat(self._OAUTH_AUTHORIZE_URL, args))

    @gen.engine
    def get_access_token(self, code, callback, grant_type='code', redirect_uri=None):
        consumer_token = self._oauth_consumer_token()
        args = {
            'client_id': consumer_token['key'],
            'client_secret': consumer_token['secret'],
        }
        if grant_type == 'refresh_token':
            args.update(
                grant_type='refresh_token',
                refresh_token=code,
            )
        elif redirect_uri:
            args.update(
                grant_type='authorization_code',
                code=code,
                redirect_uri=redirect_uri,
            )
        else:
            logging.error('Renren Get Access Token Error. redirect_uri required')
            return

        http = httpclient.AsyncHTTPClient()
        http.fetch(url_concat(self._OAUTH_ACCESS_TOKEN_URL, args),
                   callback=(yield gen.Callback('_RenrenGraphMixin.get_access_token')))
        response = yield gen.Wait('_RenrenGraphMixin.get_access_token')

        if response.error and not response.body:
            logging.warning("Error response %s fetching %s",
                            response.error, response.request.url)
            callback(None)
            return

        callback(response)
        return

    def _oauth_consumer_token(self):
        return dict(key=settings.networks['renren']['consumer_key'],
                    secret=settings.networks['renren']['consumer_secret'])


##########
# others #
##########


class FacebookAuthMixin(object):
    def authenticate_redirect(self, callback_uri=None, cancel_uri=None,
                              extended_permissions=None):
        """Authenticates/installs this app for the current user."""
        self.require_setting("facebook_api_key", "Facebook Connect")
        callback_uri = callback_uri or self.request.uri
        args = {
            "api_key": self.settings["facebook_api_key"],
            "v": "1.0",
            "fbconnect": "true",
            "display": "page",
            "next": urlparse.urljoin(self.request.full_url(), callback_uri),
            "return_session": "true",
        }
        if cancel_uri:
            args["cancel_url"] = urlparse.urljoin(
                self.request.full_url(), cancel_uri)
        if extended_permissions:
            if isinstance(extended_permissions, (unicode, bytes_type)):
                extended_permissions = [extended_permissions]
            args["req_perms"] = ",".join(extended_permissions)
        self.redirect("http://www.facebook.com/login.php?" +
                      urllib.urlencode(args))

    def authorize_redirect(self, extended_permissions, callback_uri=None,
                           cancel_uri=None):
        self.authenticate_redirect(callback_uri, cancel_uri,
                                   extended_permissions)

    def get_authenticated_user(self, callback):
        self.require_setting("facebook_api_key", "Facebook Connect")
        session = escape.json_decode(self.get_argument("session"))
        self.facebook_request(
            method="facebook.users.getInfo",
            callback=self.async_callback(
                self._on_get_user_info, callback, session),
            session_key=session["session_key"],
            uids=session["uid"],
            fields="uid,first_name,last_name,name,locale,pic_square,"
                   "profile_url,username")

    def facebook_request(self, method, callback, **args):
        self.require_setting("facebook_api_key", "Facebook Connect")
        self.require_setting("facebook_secret", "Facebook Connect")
        if not method.startswith("facebook."):
            method = "facebook." + method
        args["api_key"] = self.settings["facebook_api_key"]
        args["v"] = "1.0"
        args["method"] = method
        args["call_id"] = str(long(time.time() * 1e6))
        args["format"] = "json"
        args["sig"] = self._signature(args)
        url = "http://api.facebook.com/restserver.php?" + \
            urllib.urlencode(args)
        http = httpclient.AsyncHTTPClient()
        http.fetch(url, callback=self.async_callback(
            self._parse_response, callback))

    def _on_get_user_info(self, callback, session, users):
        if users is None:
            callback(None)
            return
        callback({
            "name": users[0]["name"],
            "first_name": users[0]["first_name"],
            "last_name": users[0]["last_name"],
            "uid": users[0]["uid"],
            "locale": users[0]["locale"],
            "pic_square": users[0]["pic_square"],
            "profile_url": users[0]["profile_url"],
            "username": users[0].get("username"),
            "session_key": session["session_key"],
            "session_expires": session.get("expires"),
        })

    def _parse_response(self, callback, response):
        if response.error:
            logging.warning("HTTP error from Facebook: %s", response.error)
            callback(None)
            return
        try:
            json = escape.json_decode(response.body)
        except Exception:
            logging.warning("Invalid JSON from Facebook: %r", response.body)
            callback(None)
            return
        if isinstance(json, dict) and json.get("error_code"):
            logging.warning("Facebook error: %d: %r", json["error_code"],
                            json.get("error_msg"))
            callback(None)
            return
        callback(json)

    def _signature(self, args):
        parts = ["%s=%s" % (n, args[n]) for n in sorted(args.keys())]
        body = "".join(parts) + self.settings["facebook_secret"]
        if isinstance(body, unicode):
            body = body.encode("utf-8")
        return hashlib.md5(body).hexdigest()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import inspect
from .errors import CommandArgumentError


class Command(object):
    allow_types = (int, float, str, unicode, bool)

    def __init__(self, func):
        self.func = func

        spec = inspect.getargspec(func)

        argnames = (spec.args or [])[:]
        defaults = list(spec.defaults or [])

        self.parameters = argnames[:len(argnames) - len(defaults)]
        self.keyword_parameters = dict(zip(argnames[- len(defaults):], defaults))

        # Check defined keyword parameters
        for k, v in self.keyword_parameters.iteritems():
            self._get_value_type(v)

        self.has_varargs = bool(spec.varargs)
        self.has_kwargs = bool(spec.keywords)

        if func.func_doc:
            doc = func.func_doc
            if '\n' in doc:
                doc = ' '.join(i.strip() for i in doc.split('\n'))
        else:
            doc = "Command '%s' in manage script" % func.func_name
        self.doc = doc

    def parse_args(self, all_args=None):
        if all_args is not None:
            all_args = all_args[:]
        else:
            all_args = sys.argv[2:]

        _kw_pos = []
        for loop, i in enumerate(all_args):
            if i.startswith('--'):
                _kw_pos.append(loop)

        if _kw_pos:
            # Check positions
            all_args_len = len(all_args)
            _fixed_kw_pos = _kw_pos + [all_args_len]
            for loop, i in enumerate(_fixed_kw_pos):
                if i == all_args_len:
                    continue
                if _fixed_kw_pos[loop + 1] - i != 2:
                    raise CommandArgumentError(
                        'Invalid arguments: %s should have one and only one value' % all_args[i])

            # Get args and keyword_args
            args = all_args[:_kw_pos[0]]
            #keyword_args = {all_args[i][2:]: all_args[i + 1] for i in _kw_pos}
            keyword_args = dict((all_args[i][2:], all_args[i + 1]) for i in _kw_pos)
        else:
            args = all_args
            keyword_args = {}

        # Check args
        if len(args) < len(self.parameters):
            raise CommandArgumentError('Arguments too little')
        else:
            if not self.has_varargs and\
               len(args) > len(self.parameters) + len(self.keyword_parameters):
                raise CommandArgumentError('Arguments too much')

        # Check keyword args
        if not self.has_kwargs:
            for i in keyword_args:
                if not i in self.keyword_parameters:
                    raise CommandArgumentError('Unknown keyword parameter %s' % i)

        # Convert keyword args type
        # NOTE As keyword arguments passed as arguments are allwed,
        # and only recognized keyword arguments will be converted,
        # so keyword arguments passed as arguments will not be converted.
        for i in keyword_args:
            if i in self.keyword_parameters:
                default = self.keyword_parameters[i]
                converted = self._convert_to_type(
                    keyword_args[i], self._get_value_type(default))
                keyword_args[i] = converted

        return args, keyword_args

    def _get_value_type(self, value):
        if isinstance(value, self.allow_types):
            typ = type(value)
        elif value is None:
            typ = None
        else:
            raise TypeError('Parameters for command function can only be type of %s' % self.allow_types)
        return typ

    def _convert_to_type(self, source, typ):
        if typ in (int, float, str, unicode):
            try:
                v = typ(source)
            except ValueError:
                raise CommandArgumentError("'%s' could not be converted to %s"
                                           % (source, typ))
        elif typ == bool:
            if source in ('True', '1'):
                v = True
            elif source in ('False', '0'):
                v = False
            else:
                raise CommandArgumentError("'%s' could not be converted to bool type" % source)
        elif typ is None:
            v = source

        return v

    def execute(self, all_args):
        args, keyword_args = self.parse_args(all_args)
        return self.func(*args, **keyword_args)


class Manager(object):
    def __init__(self, app=None):
        self.app = app
        self._commands = {}
        self._commands_list = []

    def run(self):
        all_args = sys.argv[1:]
        if not all_args or all_args[0] in ('-h', '--help'):
            self.print_usage()
            return

        if all_args[0] not in self._commands:
            self.print_small_help("'%s' is not a command" % all_args[0])
            return

        # Execute
        command = self._commands[all_args[0]]
        try:
            command.execute(all_args[1:])
        except CommandArgumentError as e:
            print 'Command execution failed: %s' % e
            self.print_command_help(command)

    def print_command_help(self, command):
        buf = []
        buf.append("'%s' usage:" % command.func.func_name)
        buf.append("  Arguments        : %s" % ','.join(command.parameters))
        buf.append("  Keyword arguments: %s" % ','.join('%s=%s' % (k, v) for k, v in command.keyword_parameters.iteritems()))
        print '\n'.join(buf)

    def print_small_help(self, hint=None):
        s = "Type '%s -h' or '%s --help' for more information"
        if hint:
            print hint + '\n' + s
        else:
            print s

    def print_usage(self, hint=None):
        """Usage format should be like:
        Lineno | Content
             1 | Script description (__doc__)
             2 | Usage: {script name} [COMMAND] [ARGUMENTS]
             3 | \n
             4 | Commands:
             5 |   cmd1               cmd1 description.
             6 |   cmd2isverylong     cmd2 description, and it is also
             7 |                      long as shit.
             7 |   cmd3               cmd3 description.
        """
        buf = []

        # Description
        if __doc__:
            buf.append(__doc__)

        # Usage
        script_name = sys.argv[0]
        buf.append('Usage: %s [COMMAND] [ARGUMENTS]' % script_name)

        buf.append('')
        buf.append('Commands:')

        # Commands
        indent_size = 2
        tab_size = 4
        doc_width = 50
        grid_len = max(len(i) for i in self._commands.keys()) + tab_size

        for name in self._commands_list:
            command = self._commands[name]
            line = ' ' * indent_size + name + ' ' * (grid_len - len(name))
            doc = command.doc
            pieces = [doc[i:i + doc_width] for i in range(0, len(doc), doc_width)]
            line += pieces[0]
            if len(pieces) > 1:
                line += '\n'
                line += '\n'.join(' ' * (grid_len + 2) + i for i in pieces[1:])

            buf.append(line)

        print '\n'.join(buf)

    def command(self, func):
        """This is a Flask-Script like decorator, provide functionality like
        @manager.command
        def foo():
            pass

        @manager.command
        def foo(first_arg, second_arg, first_option=True, second_option=3):
            pass
        """
        assert inspect.isfunction(func)

        self._commands[func.func_name] = Command(func)
        self._commands_list.append(func.func_name)

        return func

    def _init_default_commands(self):
        # TODO
        # init_completion
        pass


# Fabric like utilities

def confirm():
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import logging
from torext.utils import split_kwargs


try:
    from nose.plugins.logcapture import MyMemoryHandler
except ImportError:
    MyMemoryHandler = None


root_logger = logging.getLogger()

app_log = logging.getLogger('torext.app')
request_log = logging.getLogger('torext.request')


# borrow from tornado.options._LogFormatter.__init__
def _color(lvl):
    try:
        import curses
    except ImportError:
        curses = None
    color = False
    if curses and sys.stderr.isatty():
        try:
            curses.setupterm()
            if curses.tigetnum("colors") > 0:
                color = True
        except:
            pass
    if not color:
        return u'', u''
    # The curses module has some str/bytes confusion in
    # python3.  Until version 3.2.3, most methods return
    # bytes, but only accept strings.  In addition, we want to
    # output these strings with the logging module, which
    # works with unicode strings.  The explicit calls to
    # unicode() below are harmless in python2 but will do the
    # right conversion in python 3.
    fg_color = (curses.tigetstr("setaf") or
                curses.tigetstr("setf") or "")
    if (3, 0) < sys.version_info < (3, 2, 3):
        fg_color = unicode(fg_color, "ascii")
    colors_map = {
        logging.DEBUG: unicode(curses.tparm(fg_color, 4),  # Blue
                               "ascii"),
        logging.INFO: unicode(curses.tparm(fg_color, 2),  # Green
                              "ascii"),
        logging.WARNING: unicode(curses.tparm(fg_color, 3),  # Yellow
                                 "ascii"),
        logging.ERROR: unicode(curses.tparm(fg_color, 1),  # Red
                               "ascii"),
        'grey': unicode(curses.tparm(fg_color, 0),  # Grey
                        "ascii"),
    }
    _normal = unicode(curses.tigetstr("sgr0"), "ascii")

    return colors_map.get(lvl, _normal), _normal


FIXED_LEVELNAMES = {
    'DEBUG': 'DEBG',
    'WARNING': 'WARN',
    'ERROR': 'ERRO'
}


class BaseFormatter(logging.Formatter):
    def __init__(self,
                 fmt='%(color)s[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s',
                 datefmt='%Y-%m-%d %H:%M:%S',
                 color=False,
                 tab='  '):
        """
        a log is constituted by two part: prefix + content

        prefix is determined by `prefixfmt`, whose color depends on logging level

        content is what passed to the log method, %(message)s by default
        """
        # as origin __init__ function is very simple
        # (just store two attributes on self: _fmt & datefmt), execute it firstly
        logging.Formatter.__init__(self, datefmt=datefmt)

        self.fmt = fmt
        self.ufmt = fmt.decode('utf8')
        self.color = color
        self.tab = tab

    def _format_record(self, record):
        record.message = record.getMessage()
        record.fixed_levelname = FIXED_LEVELNAMES.get(record.levelname, record.levelname)

        if 'asctime' in self.fmt:
            record.asctime = self.formatTime(record, self.datefmt)
        if 'secs' in self.fmt:
            record.secs = record.msecs / 1000

        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)

    def format(self, record):
        """
        return log in unicode
        """
        self._format_record(record)

        record_dict = {}
        for k, v in record.__dict__.iteritems():
            if isinstance(k, str):
                k = k.decode('utf8')
            if isinstance(v, str):
                v = v.decode('utf8', 'replace')
            record_dict[k] = v

        if 'color' in self.fmt or 'end_color' in self.fmt:
            record_dict['color'], record_dict['end_color'] = _color(record.levelno)

        log = self.ufmt % record_dict

        if record.exc_text:
            if log[-1:] != u'\n':
                log += u'\n'
            log += record.exc_text.decode('utf8', 'replace')

        log = log.replace(u'\n', u'\n' + self.tab)

        return log


class BaseStreamHandler(logging.StreamHandler):
    def __init__(self, *args, **kwgs):
        _kwgs = split_kwargs(
            ('prefixfmt', 'contentfmt', 'datefmt', 'color'), kwgs)

        super(BaseStreamHandler, self).__init__(*args, **kwgs)

        self.setFormatter(BaseFormatter(**_kwgs))


HANDLER_TYPES = {
    'stream': BaseStreamHandler,
}


def set_logger(name,
               level='INFO',
               fmt=None,
               datefmt=None,
               propagate=1,
               remove_handlers=False):
    """
    This function will clear the previous handlers and set only one handler,
    which will only be StreamHandler for the logger.

    This function is designed to be able to called multiple times in a context.

    Note that if a logger has no handlers, it will be added a handler automatically when it is used.
    """
    logger = logging.getLogger(name)
    logger.setLevel(getattr(logging, level))
    logger.propagate = propagate

    if remove_handlers:
        logger.handlers = []
        return

    handler = None
    for h in logger.handlers:
        if isinstance(h, logging.StreamHandler):
            # use existing instead of clean and create
            handler = h
            break
    if not handler:
        handler = logging.StreamHandler()
        logger.addHandler(handler)

    formatter_kwgs = {}
    for i in ('fmt', 'datefmt'):
        if locals()[i] is not None:
            formatter_kwgs[i] = locals()[i]
    handler.setFormatter(BaseFormatter(**formatter_kwgs))


def set_loggers(loggers):
    for name, config in loggers.iteritems():
        set_logger(name, **config)


def set_nose_formatter(logging_options):
    if not MyMemoryHandler:
        return

    formatter_kwgs = {}
    for i in ('fmt', 'datefmt'):
        if i in logging_options:
            formatter_kwgs[i] = logging_options[i]
    formatter = BaseFormatter(**formatter_kwgs)

    nose_handler = None
    for h in root_logger.handlers:
        if isinstance(h, MyMemoryHandler):
            h.setFormatter(formatter)
            nose_handler = h
            break

    if nose_handler:
        root_logger.handlers = [nose_handler, ]


#############
#  loggers  #
#############
# 1. test - propagate 0
# 2. system - propagate 1 - for seperately output system level logs

#test_logger = logging.getLogger('test')
#test_logger.propagate = 0
#test_logger.setLevel(logging.INFO)
#test_logger.handlers = []


if __name__ == '__main__':
    def test_all():
        root_logger = logging.getLogger()
        # root_logger.setLevel(logging.INFO)
        # streamHandler = logging.StreamHandler()
        # streamHandler.setFormatter(BaseFormatter(color=True))
        # root_logger.addHandler(streamHandler)
        set_logger('', level=logging.DEBUG, color=True)

        root_logger.debug('bug..')
        root_logger.info('hello')
        root_logger.warning('\nholy a shit')
        try:
            tuple()[0]
        except Exception, e:
            root_logger.error(e, exc_info=True)

        # this logger's log will be handled only once, due to the bool False value
        # of testLogger's attribute `propagate`
        # testLogger.addHandler(logging.StreamHandler())
        # testLogger.info('my name is testLogger')

        # this logger's log will be handled twice, one by its self, with uncustomized StreamHandler instance,
        # the other by rootLogger, which is the parent of otherLogger, see quote below::
        #
        # http://docs.python.org/howto/logging.html#loggers
        #
        #     " Child loggers propagate messages up to the handlers associated
        #     with their ancestor loggers. Because of this, it is unnecessary to
        #     define and configure handlers for all the loggers an application
        #     uses. It is sufficient to configure handlers for a top-level
        #     logger and create child loggers as needed. (You can, however, turn
        #     off propagation by setting the propagate attribute of a logger to
        #     False.) "
        otherLogger = logging.getLogger('other')
        fmter = logging.Formatter(fmt='%(message)s %(msecs)s')
        hdr = logging.StreamHandler()
        hdr.setFormatter(fmter)
        otherLogger.addHandler(hdr)
        otherLogger.info('here is otherLogger')

        # and this logger, its log will ofcoursely be handled three times
        otherBabyLogger = logging.getLogger('other.baby')
        otherBabyLogger.addHandler(hdr)
        otherBabyLogger.info('here is otherBabyLogger')

    test_all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import re
import copy
import uuid
import datetime
import functools
from tornado.escape import _unicode, json_decode
from torext.errors import ValidationError, ParamsInvalidError


# don't know where to find the <type '_sre.SRE_Pattern'>
_pattern_class = type(re.compile(''))


class Field(object):
    name = None
    with_choices = True

    def __init__(self, description=None, key=None, required=False, length=None, choices=None, default=None, null=True):
        self.description = description  # default message
        self.required = required  # used with ParamSet
        self.choices = choices
        self.key = key
        self.default = default
        self.null = null

        self.min_length = None
        assert length is None or isinstance(length, (int, tuple))
        if isinstance(length, int):
            assert length > 0
        if isinstance(length, tuple):
            assert len(length) == 2 and length[0] > 0
            if length[1] == 0:
                self.min_length = length[0]
            else:
                assert length[1] > length[0]
        self.length = length

    #@property
    #def name(self):
        #raise NotImplementedError

    def raise_exc(self, error_message=None):
        raise ValidationError(self.description, error_message)

    def _validate_length(self, value):
        length = self.length
        value_len = len(value)

        if isinstance(length, int):
            if value_len != length:
                self.raise_exc('Length of value should be %s, but %s' % (length, value_len))
        else:
            if self.min_length:
                if value_len < self.min_length:
                    self.raise_exc('Length of value should be larger than %s' % self.min_length)
            else:
                min, max = length
                if value_len < min or value_len > max:
                    self.raise_exc('Length should be >= %s and <= %s, but %s' % (min, max, value_len))
        return value

    def _validate_choices(self, value):
        if not value in self.choices:
            raise ValidationError('value "%s" is not one of %s' % (value, self.choices))
        return value

    def _validate_type(self, value):
        """Override this method to implement type specified validation"""
        return value

    def validate(self, value):
        # If null is allowed, skip other validates
        if not value:
            if self.null:
                return value
            else:
                raise ValidationError('empty value is not allowed')

        if self.length:
            self._validate_length(value)

        value = self._validate_type(value)

        # Validate choices after type, so that the value has been converted
        if self.with_choices and self.choices:
            self._validate_choices(value)

        return value

    def __get__(self, owner, cls):
        return owner.data.get(self.key, self.default)

    def __set__(self, owner, value):
        raise AttributeError('You can not set value to a param field')

    def spawn(self, **kwargs):
        new = copy.copy(self)
        new.__dict__.update(kwargs)
        #for k, v in kwargs.iteritems():
            #setattr(new, k, v)
        return new


class RegexField(Field):
    def __init__(self, *args, **kwgs):
        # assume pattern is a raw string like r'\n'
        if 'pattern' in kwgs:
            pattern = kwgs.pop('pattern')
            self.regex = re.compile(pattern)
        assert hasattr(self, 'regex'),\
            'regex should be set if no keyword argument pattern passed'
        assert isinstance(self.regex, _pattern_class),\
            'regex should be a compiled pattern'

        super(RegexField, self).__init__(*args, **kwgs)

    def _validate_type(self, value):
        # Equate the type of regex pattern and the checking value
        pattern_type = type(self.regex.pattern)
        c_value = value
        if not isinstance(value, pattern_type):
            try:
                c_value = pattern_type(value)
            except Exception, e:
                self.raise_exc(
                    'value could not be converted into type "%s" of regex pattern'
                    ', error: %s' % (pattern_type, e))
        if not self.regex.search(c_value):
            self.raise_exc('regex pattern (%s, %s) is not match with value "%s"' %
                           (self.regex.pattern, self.regex.flags, c_value))
        return value


class WordField(RegexField):
    """
    >>> v = WordField('should not contain punctuations')
    >>> s = 'oh123,'
    >>> v.validate(s)
    ValidationError: should not contain punctuations
    """

    regex = re.compile(r'^[\w]+$')


# take from Django
EMAIL_REGEX = re.compile(
    r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"  # dot-atom
    r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-011\013\014\016-\177])*"'  # quoted-string
    r')@(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?$', re.IGNORECASE)  # domain


class EmailField(RegexField):
    regex = EMAIL_REGEX


URL_REGEX = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


class URLField(RegexField):
    regex = URL_REGEX


class IntegerField(Field):
    def __init__(self, *args, **kwargs):
        min = kwargs.pop('min', None)
        max = kwargs.pop('max', None)
        if min is not None:
            assert isinstance(min, int)
        if max is not None:
            assert isinstance(max, int)
        if min is not None and max is not None:
            assert min <= max

        self.min = min
        self.max = max

        super(IntegerField, self).__init__(*args, **kwargs)

    def _validate_type(self, value):
        try:
            value = int(value)
        except (ValueError, TypeError):
            self.raise_exc('could not convert value "%s" into int type' % value)

        if self.min:
            if value < self.min:
                self.raise_exc('value is too small, min %s' % self.min)
        if self.max:
            if value > self.max:
                self.raise_exc('vaule is too big, max %s' % self.max)

        return value


# TODO
class FloatField(Field):
    pass


# TODO
class DateField(Field):
    def __init__(self, *args, **kwargs):
        datefmt = kwargs.pop('datefmt', None)
        assert datefmt, '`datefmt` argument should be passed for DateField'
        self.datefmt = datefmt
        super(DateField, self).__init__(*args, **kwargs)

    def _validate_type(self, value):
        try:
            value = datetime.datetime.strptime(value, self.datefmt)
        except ValueError:
            self.raise_exc('Could not convert %s to datetime object by format %s' % (value, self.datefmt))
        return value


class ListField(Field):
    with_choices = False

    def __init__(self, *args, **kwargs):
        self.item_field = kwargs.pop('item_field', None)
        super(ListField, self).__init__(*args, **kwargs)

    def _validate_type(self, value):
        if not isinstance(value, list):
            raise ValidationError('Not a list')

        if self.item_field:
            formatted_value = []
            for i in value:
                formatted_value.append(self.item_field.validate(i))
            value = formatted_value

        bad_values = []
        if self.choices:
            for i in value:
                if not i in self.choices:
                    bad_values.append(i)
        if bad_values:
            raise ValidationError('%s is/are not allowed' % bad_values)

        return value


class UUIDField(Field):
    def _validate_type(self, value):
        try:
            return uuid.UUID(value)
        except ValueError, e:
            raise ValidationError('Invalid uuid string: %s' % e)


class ParamSetMeta(type):
    def __new__(cls, name, bases, attrs):
        fields = {}
        for k, v in attrs.iteritems():
            if isinstance(v, Field):
                v.name = k
                if not v.key:
                    v.key = k
                fields[k] = v
        attrs['_fields'] = fields
        return type.__new__(cls, name, bases, attrs)


# TODO user-define check function
class ParamSet(object):
    """
    item in `errors` could be:
        tuple: (key, ValidationError)
        exception: ValidationError
    """
    __metaclass__ = ParamSetMeta
    __datatype__ = 'form'  # or 'json'

    @classmethod
    def keys(cls):
        return [i.key for i in cls._fields.itervalues()]

    def __init__(self, **kwargs):
        self._raw_data = {}
        # handler.request.arguments, utf-8 values
        for k in kwargs:
            if isinstance(kwargs[k], list):
                self._raw_data[k] = map(_unicode, kwargs[k])
            else:
                self._raw_data[k] = _unicode(kwargs[k])

        self.data = {}
        self.errors = []

        self.validate()

    def validate(self):
        for name, field in self.__class__._fields.iteritems():
            key = field.key
            if key in self._raw_data:

                value = self._raw_data[key]

                if not isinstance(field, ListField) and isinstance(value, list):
                    # tornado request.arguments hack, value may be a list,
                    # only use the first one
                    value = value[0]

                try:
                    value = field.validate(value)
                    func_name = 'validate_' + name
                    if hasattr(self, func_name):
                        value = getattr(self, func_name)(value)
                        assert value is not None, 'Forget to return value after validation? Or this is caused by your explicitly returns None, which is not allowed in the mechanism.'
                except ValidationError, e:
                    self.errors.append((key, e))
                else:
                    self.data[key] = value
            else:
                if field.required:
                    try:
                        field.raise_exc('%s is required' % key)
                    except ValidationError, e:
                        self.errors.append((key, e))

        for attr_name in dir(self):
            if attr_name.startswith('validate_') and\
                    attr_name[len('validate_'):] not in self._fields:
                try:
                    getattr(self, attr_name)()
                except ValidationError, e:
                    self.errors.append(e)

    def kwargs(self, *args):
        d = {}
        for k in args:
            v = getattr(self, k)
            if v is not None:
                d[k] = v
        return d

    def has(self, name):
        return name in self.data

    def to_dict(self, include_none=False):
        """Convert the ParamSet instance to a dict, if `include_none` is True,
        the result will represent both data and schema.
        """
        d = {}
        for f in self.__class__._fields.itervalues():
            value = getattr(self, f.name)
            if value is not None or (value is None and include_none):
                d[f.key] = value
        return d

    def __str__(self):
        return self.__unicode__().encode('utf8')

    def __unicode__(self):
        return u'<%s: %s; errors=%s>' % (self.__class__.__name__,
                                         u','.join([u'%s=%s' % (k, v) for k, v in self.data.iteritems()]),
                                         self.errors)

    @classmethod
    def validation_required(cls, method):
        @functools.wraps(method)
        def wrapper(hdr, *args, **kwgs):
            if 'json' == cls.__datatype__:
                try:
                    arguments = json_decode(hdr.request.body)
                except Exception, e:
                    raise ParamsInvalidError('JSON decode failed: %s' % e)
            else:
                arguments = hdr.request.arguments
            params = cls(**arguments)
            if params.errors:
                raise ParamsInvalidError(params.errors)
            hdr.params = params
            return method(hdr, *args, **kwgs)
        return wrapper


def define_params(kwargs):
    param_class = type('AutoCreatedParams', (ParamSet, ), kwargs)
    return param_class.validation_required


def simple_params(method):
    @functools.wraps(method)
    def wrapper(hdr, *args, **kwgs):
        if 'json' == getattr(method.__class__, '__datatype__', None):
            try:
                params = json_decode(hdr.request.body)
            except Exception, e:
                raise ParamsInvalidError('JSON decode failed: %s' % e)
        else:
            params = dict((k, v[0]) for k, v in hdr.request.arguments.iteritems())

        hdr.params = params
        return method(hdr, *args, **kwgs)
    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    import jinja2
except ImportError:
    print 'jinja2 is not installed, skip testing'
    from nose.plugins.skip import SkipTest
    raise SkipTest


def test_jinja2_render():
    from torext import settings
    from torext.handlers.base import jinja2_render

    # Fake settings
    settings['PROJECT'] = 'torext'
    settings['TEMPLATE_PATH'] = 'test/jinja2'

    context = {
        'name': 'Jinja2'
    }
    rendered = jinja2_render('template.html', **context)

    assert rendered == jinja2_render('rendered.html')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
#import unittest
import tempfile
try:
    import simplejson as json
except ImportError:
    import json

from torext.app import TorextApp
from torext.handlers.base import BaseHandler
from torext import errors
from torext.utils import generate_cookie_secret


EXC_MSG_0 = 'not pass'
EXC_MSG_1 = 'invalid'
EXC_MSG_2 = 'need proper index'

JSON_DICT = {'eva': 'evangelion'}

FILE_CONTENT = """
Mari Illustrious Makinami
Voiced by: Maaya Sakamoto (Japanese), Trina Nishimura (English)
"""

SIGNED_NAME = 'EVA-02'
SIGNED_RAW_VALUE = 'Asuka Langley Soryu'

BUF_DICT = {
    '01': 'You Are (Not) Alone',
    '02': 'You Can (Not) Advance',
    '03': 'You Can (Not) Redo'
}


def same_dict(d1, d2):
    return set(d1.keys()) == set(d2.keys()) and\
        not [i for i in d1 if d1[i] != d2[i]]


app = TorextApp()
app.settings['COOKIE_SECRET'] = generate_cookie_secret()


@app.route('/')
class HomeHandler(BaseHandler):
    EXCEPTION_HANDLERS = {
        errors.AuthenticationNotPass: '_handle_401',
        (errors.ValidationError, IndexError): '_handle_400'
    }

    def get(self):
        exc = int(self.get_argument('exc'))
        if exc == 0:
            raise errors.AuthenticationNotPass(EXC_MSG_0)
        elif exc == 1:
            raise errors.ValidationError(EXC_MSG_1)
        elif exc == 2:
            raise IndexError(EXC_MSG_2)

    def _handle_401(self, e):
        # unauthorized
        self.set_status(401)
        self.write(str(e))

    def _handle_400(self, e):
        # bad request
        self.set_status(400)
        self.write(str(e))


@app.route('/file')
class FileHandler(BaseHandler):
    def get(self):
        _, fname = tempfile.mkstemp()
        with open(fname, 'w') as f:
            f.write(FILE_CONTENT)

        self.write_file(fname, mime_type='text/plain')
        os.remove(fname)


@app.route('/cookie')
class CookieHandler(BaseHandler):
    def get(self):
        signed_value = self.create_signed_value(SIGNED_NAME, SIGNED_RAW_VALUE)
        self.set_cookie(SIGNED_NAME, signed_value)

    def post(self):
        print 'request header', self.request.headers
        print 'request cookies', self.request.cookies
        cookie_value = self.get_cookie(SIGNED_NAME)
        self.write(self.decode_signed_value(SIGNED_NAME, cookie_value))


@app.route('/prepare')
class PrepareHandler(BaseHandler):
    PREPARES = ['01', '02', '03']

    def get(self):
        self.write_json(self.prepare_buf)

    def prepare_01(self):
        self.prepare_buf = {}
        self.prepare_buf['01'] = BUF_DICT['01']

    def prepare_02(self):
        self.prepare_buf['02'] = BUF_DICT['02']

    def prepare_03(self):
        self.prepare_buf['03'] = BUF_DICT['03']


@app.route('/json')
class JsonHandler(BaseHandler):
    def get(self):
        if self.get_argument('write_json', None):
            self.write_json(JSON_DICT, code=201, headers={'EVA-01': 'Shinji Ikari'})
        else:
            self.write(self.json_encode(JSON_DICT))

    def post(self):
        print 'headers', self.request.headers
        data = self.get_argument('data')
        d = self.json_decode(data)
        if d == JSON_DICT:
            self.set_status(200)
        else:
            self.set_status(400)


class BaseHandlerRequestTest(app.TestCase):
    def test_handle_request_exception(self):
        resp = self.c.get('/', {'exc': 0})
        assert resp.code == 401 and resp.body == EXC_MSG_0

        resp = self.c.get('/', {'exc': 1})
        assert resp.code == 400 and resp.body == EXC_MSG_1

        resp = self.c.get('/', {'exc': 2})
        assert resp.code == 400 and resp.body == EXC_MSG_2

    def test_file_write(self):
        resp = self.c.get('/file')
        print repr(resp.body)
        assert resp.body == FILE_CONTENT

        assert  resp.headers.get('Content-Type') == 'text/plain'
        assert 'Last-Modified' in resp.headers
        assert 'ETag' in resp.headers

    def test_decode_signed_value(self):
        resp = self.c.get('/cookie')
        print resp.cookies
        resp_post = self.c.post('/cookie', cookies=resp.cookies)
        print repr(resp_post.body)
        assert resp_post.body == SIGNED_RAW_VALUE

    def test_prepare(self):
        resp = self.c.get('/prepare')
        assert json.loads(resp.body) == BUF_DICT

    def test_json_write(self):
        resp = self.c.get('/json?write_json=1')
        assert resp.code == 201
        assert resp.headers.get('EVA-01') == 'Shinji Ikari'

        assert json.loads(resp.body) == JSON_DICT

    def test_dump_dict(self):
        resp = self.c.get('/json')
        print resp.body
        assert json.loads(resp.body) == JSON_DICT

    def test_parse_json(self):
        resp = self.c.post('/json', data={'data': json.dumps(JSON_DICT)})
        assert resp.code == 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
errors:

    ConnectionError

    ValidationError

    AuthenticationNotPass

    ObjectNotFound

    MultiObjectsReturned

    ParamsInvalidError
"""


# import inspect


class TorextException(Exception):
    pass


class URLRouteError(TorextException):
    """
    error in router
    """
    pass


class ValidationError(TorextException):
    """
    error occur when validating values
    """
    def __init__(self, description=None, error_message=None):
        self.description = description
        self.error_message = error_message

    def __str__(self):
        #return '%s (%s)' % (self.description, self.error_message) if self.error_message else self.description
        return self.description or self.error_message

    def __repr__(self):
        return self.__str__()


class AuthenticationNotPass(TorextException):
    pass


##
# TODO add `errors` to its attribute
class ParamsInvalidError(TorextException):
    def __init__(self, error_s):
        if isinstance(error_s, list):
            self.errors = error_s
        else:
            self.errors = [error_s, ]

    def __str__(self):
        return 'Invalid params: %s' % self.errors


class CommandArgumentError(TorextException):
    pass


class OperationNotAllowed(TorextException):
    pass


class OperationFailed(TorextException):
    pass


class SettingsError(TorextException):
    pass


class ArgsParseError(TorextException):
    pass


class DatabaseError(TorextException):
    pass


class JSONDecodeError(TorextException):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from torext.log import set_logger
from nose.tools import with_setup


def test_logging():
    msgs = [
        '中文 utf8',
        u'中文 gbk'.encode('gbk'),
        u'中文 unicode',
        '始める utf8',
        u'始める shift_jis'.encode('shift_jis'),
        u'始める unicode'
    ]
    for i in msgs:
        yield do_logging, i


def logging_setup():
    set_logger('')


def logging_teardown():
    logging.getLogger().handlers = []


@with_setup(logging_setup, logging_teardown)
def do_logging(msg):
    logging.info(msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import httplib
import datetime
import email.utils
import hashlib
import mimetypes
import os.path
import stat
import time

import tornado.web
import tornado.locale
from tornado.web import HTTPError
from tornado.escape import utf8

from torext import settings, errors
from torext.log import app_log
from torext.app import TorextApp
from torext.utils import raise_exc_info


def log_response(handler):
    """
    Acturally, logging response is not a server's responsibility,
    you should use http tools like Chrome Developer Tools to analyse the response.

    Although this function and its setting(LOG_RESPONSE) is not recommended to use,
    if you are laze as I was and working in development, nothing could stop you.
    """
    content_type = handler._headers.get('Content-Type', None)
    headers_str = handler._generate_headers()
    block = 'Response Infomations:\n' + headers_str.strip()

    if content_type and ('text' in content_type or 'json' in content_type):
        limit = 0
        if 'LOG_RESPONSE_LINE_LIMIT' in settings:
            limit = settings['LOG_RESPONSE_LINE_LIMIT']

        def cut(s):
            if limit and len(s) > limit:
                return [s[:limit]] + cut(s[limit:])
            else:
                return [s]

        body = ''.join(handler._write_buffer)
        lines = []
        for i in body.split('\n'):
            lines += ['| ' + j for j in cut(i)]
        block += '\nBody:\n' + '\n'.join(lines)
    app_log.info(block)


def log_request(handler):
    """
    Logging request is opposite to response, sometime its necessary,
    feel free to enable it.
    """
    block = 'Request Infomations:\n' + _format_headers_log(handler.request.headers)

    if handler.request.arguments:
        block += '+----Arguments----+\n'
        for k, v in handler.request.arguments.iteritems():
            block += '| {0:<15} | {1:<15} \n'.format(repr(k), repr(v))

    app_log.info(block)


def _format_headers_log(headers):
    # length of '+-...-+' is 19
    block = '+-----Headers-----+\n'
    for k, v in headers.iteritems():
        block += '| {0:<15} | {1:<15} \n'.format(k, v)
    return block


class BaseHandler(tornado.web.RequestHandler):
    """
    Request
        header:
        body:

    Response
        status code: 200(成功), 400(参数异常), 403(不成功), 404(web,未找到), 500(服务器异常)
        header:
        body:
    """
    EXCEPTION_HANDLERS = None

    PREPARES = []

    def _exception_default_handler(self, e):
        """This method is a copy of tornado.web.RequestHandler._handle_request_exception
        """
        if isinstance(e, HTTPError):
            if e.log_message:
                format = "%d %s: " + e.log_message
                args = [e.status_code, self._request_summary()] + list(e.args)
                app_log.warning(format, *args)
            if e.status_code not in httplib.responses:
                app_log.error("Bad HTTP status code: %d", e.status_code)
                self.send_error(500, exc_info=sys.exc_info())
            else:
                self.send_error(e.status_code, exc_info=sys.exc_info())
        else:
            app_log.error("Uncaught exception %s\n%r", self._request_summary(),
                             self.request, exc_info=True)
            self.send_error(500, exc_info=sys.exc_info())

    def _handle_request_exception(self, e):
        """This method handle HTTPError exceptions the same as how tornado does,
        leave other exceptions to be handled by user defined handler function
        maped in class attribute `EXCEPTION_HANDLERS`

        Common HTTP status codes:
            200 OK
            301 Moved Permanently
            302 Found
            400 Bad Request
            401 Unauthorized
            403 Forbidden
            404 Not Found
            405 Method Not Allowed
            500 Internal Server Error

        It is suggested only to use above HTTP status codes
        """
        handle_func = self._exception_default_handler
        if self.EXCEPTION_HANDLERS:
            for excs, func_name in self.EXCEPTION_HANDLERS.iteritems():
                if isinstance(e, excs):
                    handle_func = getattr(self, func_name)
                    break

        handle_func(e)
        if not self._finished:
            self.finish()

    @property
    def app(self):
        return TorextApp.current_app

    @property
    def db(self):
        """Rewrite this method to implement the default database connection object,
        eg. a sqlalchemy session

        Other names like ``mongodb``, ``redis``, ``memcache`` could also be used in this way.
        """
        raise NotImplementedError

    @property
    def json_decode(self):
        return self.app.json_decoder

    @property
    def json_encode(self):
        return self.app.json_encoder

    def flush(self, *args, **kwgs):
        """
        Before `RequestHandler.flush` was called, we got the final _write_buffer.

        This method will not be called in wsgi mode
        """
        if settings['LOG_RESPONSE'] and not self._status_code == 500:
            log_response(self)

        super(BaseHandler, self).flush(*args, **kwgs)

    def write_json(self, chunk, code=None, headers=None):
        """A convenient method that binds `chunk`, `code`, `headers` together

        chunk could be any type of (str, dict, list)
        """
        assert chunk is not None, 'None cound not be written in write_json'
        self.set_header("Content-Type", "application/json; charset=UTF-8")
        if isinstance(chunk, dict) or isinstance(chunk, list):
            chunk = self.json_encode(chunk)

        # convert chunk to utf8 before `RequestHandler.write()`
        # so that if any error occurs, we can catch and log it
        try:
            chunk = utf8(chunk)
        except Exception:
            app_log.error('chunk encoding error, repr: %s' % repr(chunk))
            raise_exc_info(sys.exc_info())

        self.write(chunk)

        if code:
            self.set_status(code)

        if headers:
            for k, v in headers.iteritems():
                self.set_header(k, v)

    def write_file(self, file_path, mime_type=None):
        """Copy from tornado.web.StaticFileHandler
        """
        if not os.path.exists(file_path):
            raise HTTPError(404)
        if not os.path.isfile(file_path):
            raise HTTPError(403, "%s is not a file", file_path)

        stat_result = os.stat(file_path)
        modified = datetime.datetime.fromtimestamp(stat_result[stat.ST_MTIME])

        self.set_header("Last-Modified", modified)

        if not mime_type:
            mime_type, _encoding = mimetypes.guess_type(file_path)
        if mime_type:
            self.set_header("Content-Type", mime_type)

        # Check the If-Modified-Since, and don't send the result if the
        # content has not been modified
        ims_value = self.request.headers.get("If-Modified-Since")
        if ims_value is not None:
            date_tuple = email.utils.parsedate(ims_value)
            if_since = datetime.datetime.fromtimestamp(time.mktime(date_tuple))
            if if_since >= modified:
                self.set_status(304)
                return

        with open(file_path, "rb") as file:
            data = file.read()
            hasher = hashlib.sha1()
            hasher.update(data)
            self.set_header("Etag", '"%s"' % hasher.hexdigest())
            self.write(data)

    def decode_signed_value(self, name, value, max_age_days=None):
        """Do what `RequestHandler.get_secure_cookie` does(when `value` is not None),
        but with a more friendly name

        What opposite to it is `RequestHandler.create_signed_value`
        """
        kwgs = {}
        if max_age_days is not None:
            kwgs['max_age_days'] = max_age_days
        return tornado.web.decode_signed_value(settings['COOKIE_SECRET'], name, value, **kwgs)

    def prepare(self):
        """Behaves like a middleware between raw request and handling process,

        If `PREPARES` is defined on handler class, which should be
        a list, for example, ['auth', 'context'], method whose name
        is constitute by prefix '_prepare_' and string in this list
        will be executed by sequence. In this example, those methods are
        `_prepare_auth` and `_prepare_context`
        """
        if settings['LOG_REQUEST']:
            log_request(self)

        for i in self.PREPARES:
            getattr(self, 'prepare_' + i)()
            if self._finished:
                return

    def render_string(self, template_name, **kwargs):
        """This method was rewrited to support multiple template engine
        (Determine by `TEMPLATE_ENGINE` setting, could be `tornado` and `jinja2`),
        it will only affect on template rendering process, ui modules feature,
        which is mostly exposed in `render` method, is kept to be used as normal.
        """
        if 'tornado' == settings['TEMPLATE_ENGINE']:
            return super(BaseHandler, self).render_string(template_name, **kwargs)
        elif 'jinja2' == settings['TEMPLATE_ENGINE']:
            return jinja2_render(template_name, **kwargs)
        else:
            raise errors.SettingsError(
                '%s is not a supported TEMPLATE_ENGINE, should be `tornado` or `jinja2`'
                % settings['TEMPLATE_ENGINE'])


_jinja2_env = None


def jinja2_render(template_name, **kwargs):
    from jinja2 import Environment, PackageLoader

    global _jinja2_env
    if not _jinja2_env:
        _jinja2_env = Environment(loader=PackageLoader(settings['PROJECT'], settings['TEMPLATE_PATH']))

    template = _jinja2_env.get_template(template_name)
    return template.render(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# variables in this module are essential basements of settings,
# the priority sequence of base_settings.py, settings.py(in project), and cmd options is::
#   1. commandline arguments
#   2. settings.py
#   3. base_settings.py


#############
# essential #
#############

PROJECT = None

LOCALE = 'en_US'

PROCESSES = 1

PORT = 8000

DEBUG = True

TESTING = False

LOGGERS = {
    '': {
        'level': 'INFO',
        'fmt': '%(color)s[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s',
        #'fmt': '[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d] %(message)s',
        'datefmt': '%Y-%m-%d %H:%M:%S',
    }
}

LOG_REQUEST = False

LOG_RESPONSE = False

LOG_RESPONSE_LINE_LIMIT = 0

TIME_ZONE = 'Asia/Shanghai'

STATIC_PATH = 'static'

TEMPLATE_PATH = 'template'

TEMPLATE_ENGINE = 'tornado'

LOGGING_IGNORE_URLS = [
    '/favicon.ico',
]


############
# optional #
############

# COOKIE_SECRET = 'P0UTa5iuRaaVlV8QZF2uVR7hHwTOSkQhg2Fol18OKwc='

# SECURE_COOKIE = ''
# SECURE_HEADER = ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from torext.utils import SingletonMixin
from torext.errors import SettingsError
from torext.log import set_loggers


class Settings(dict, SingletonMixin):
    """
    Philosophy was borrowed from django.conf.Settings

    As there are just few necessary values involved by tornado.options.options,
    (in httpclient and testing), and options.define is complicate and not convenient to use,
    Settings is created to replace tornado.options.options.

    by import torext module, a Settings object will be instanced and stored globally,
    then it can be involved in any place like this:
    >>> import torext
    >>> print torext.settings
    or
    >>> from torext import settings
    >>> print settings

    getting value from settings is like from a normal dict:
    >>> settings['DEBUG']
    True
    >>> settings.get('PORT')
    8000
    >>> settings.get('WTF', None)
    None

    notice that you can use lower case word to get or set the value:
    >>> settings['debug'] is settings.get('DEBUG')
    True
    >>> settings['port'] = 8765
    >>> settings['PORT']
    8765

    TODO require_setting
    """
    def __init__(self):
        """
        Setting definitions in base_settings are indispensable
        """
        self._callbacks = {}
        self.add_key_callback('LOGGERS', set_loggers)

        from torext import base_settings

        for i in dir(base_settings):
            if not i.startswith('_'):
                self[i] = getattr(base_settings, i)

        self._module = None

    def __getitem__(self, key):
        try:
            return super(Settings, self).__getitem__(key)
        except KeyError:
            try:
                return super(Settings, self).__getitem__(key.lower())
            except KeyError:
                raise SettingsError('Key "%s" is not defined in settings' % key)

    def __setitem__(self, key, value):
        for i in key:
            if i != i.upper():
                raise SettingsError('Key "%s" is not allowed, you should always define'
                                    ' UPPER CASE VARIABLE as setting' % key)
        key_upper = key.upper()
        super(Settings, self).__setitem__(key_upper, value)
        if key_upper in self._callbacks:
            self._callbacks[key](value)

    def __str__(self):
        return '<Settings. %s >' % dict(self)

    def add_key_callback(self, key, callback):
        self._callbacks[key] = callback

settings = Settings.instance()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import unittest
from torext.app import TorextApp
from torext.handlers import BaseHandler
from torext.testing import AppTestCase


GET_RESULT = 'get ok'
POST_RESULT = 'post ok'


def make_app():
    app = TorextApp()

    @app.route('/')
    class HomeHdr(BaseHandler):
        def get(self):
            self.write(GET_RESULT)

        def post(self):
            self.write(POST_RESULT)

    @app.route('/withdata')
    class WithdataHdr(BaseHandler):
        def get(self):
            self.write(self.get_argument('p'))

        def post(self):
            self.write(self.get_argument('d'))

    @app.route('/header')
    class HeaderHdr(BaseHandler):
        def get(self):
            header_name = self.get_argument('h')
            header_value = self.request.headers.get(header_name)
            self.set_header(header_name, header_value)
            self.write(header_value)

    # although this is not needed, it's good to be set explicitly
    app.update_settings({
        'TESTING': True
    })
    return app


class CaseMixin(object):
    def test_get(self):
        rv = self.c.get('/')
        assert rv.body == GET_RESULT

    def test_post(self):
        rv = self.c.post('/')
        assert rv.body == POST_RESULT

    def test_get_params(self):
        p = 'fly me to the moon'
        rv = self.c.get('/withdata', {'p': p})
        assert rv.body == p

    def test_post_data(self):
        d = 'in other words'
        rv = self.c.post('/withdata', {'d': d})
        assert rv.body == d

    def test_header_change(self):
        h = 'darling kiss me'
        name = 'Torext-Special'
        rv = self.c.get('/header', {'h': name}, headers={name: h})

        assert rv.headers.get(name) == h
        assert rv.body == h


class BasicTestCase(unittest.TestCase, CaseMixin):
    def setUp(self):
        app = make_app()

        self.c = app.test_client()

    def tearDown(self):
        self.c.close()


class AppTestCaseTest(AppTestCase, CaseMixin):
    def get_client(self):
        return make_app().test_client()


app = make_app()


class AppGeneratedTestCaseTest(app.TestCase, CaseMixin):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from torext.errors import CommandArgumentError
from torext.script import Command

from nose.tools import assert_raises


def test_func_parse():
    def pri(a, b, c=True, d=3):
        print 'pri func'
        return

    c = Command(pri)
    assert c.parameters == ['a', 'b']
    assert c.keyword_parameters == {'c': True, 'd': 3}

    with assert_raises(CommandArgumentError):
        c.parse_args([])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a'])

    c.parse_args(['v a', 'v b'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c'])

    c.parse_args(['v a', 'v b', '--c', '1'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '1', '2'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '1', '2', '--d'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '--d'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '--d', '3'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '--d', '3'])

    c.parse_args(['v a', 'v b', '--c', '1', '--d', '3'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '1', '--d', '3', '4'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--x', '1'])

    with assert_raises(CommandArgumentError):
        c.parse_args(['v a', 'v b', '--c', '1', '--x', '2'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Features:
    * scope control for session in tornado environment
    * signal in commit process (use new api: orm.events)
    * signal in connection process

Finished:
    * bind all frequently-used classes & methods to SQLAlchemy object
    * Model class for creating new models
    * customizable Query object
"""

import os
import logging
import functools
import sqlalchemy
import sqlalchemy.orm
from sqlalchemy.engine.url import make_url
from sqlalchemy.orm import sessionmaker, scoped_session, Query
from sqlalchemy.ext.declarative import declarative_base, declared_attr

from torext.app import TorextApp


logger = logging.getLogger('sqlalchemy')
logger.propagate = 0


def _make_table(db):
    def _Table(*args, **kwargs):
        """
        if passing (name, column, ..), existing metadata will be added to args,
        return Table(name, metadata, column, ..)
        """
        if len(args) > 1 and isinstance(args[1], db.Column):
            args = (args[0], db.metadata) + args[1:]
        return sqlalchemy.Table(*args, **kwargs)
    return _Table


def _set_default_query_class(d):
    if 'query_class' not in d:
        d['query_class'] = BaseQuery


def _wrap_with_default_query_class(fn):
    @functools.wraps(fn)
    def newfn(*args, **kwargs):
        _set_default_query_class(kwargs)
        if "backref" in kwargs:
            backref = kwargs['backref']
            if isinstance(backref, basestring):
                backref = (backref, {})
            _set_default_query_class(backref[1])
        return fn(*args, **kwargs)
    return newfn


class BaseQuery(Query):
    """
    Add any function on this class, so that it can be called on SomeModel.query
    """
    pass


class _Model(object):
    query = None

    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()

    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class SQLAlchemy(object):
    """
    Usage:

    1. with app
    # database.py
    >>> from torext.sql import SQLAlchemy
    >>> db = SQLAlchemy()

    # app.py
    >>> from myproject.database import db
    >>> db.init_app(app)

    2. without app
    >>> from torext.sql import SQLAlchemy
    >>> db = SQLAlchemy('sqlite://')
    """

    def __init__(self, app=None, uri=None, config=None):
        self._engine = None

        # session is created without an engine,
        # it will be binded with engine later by uri argument
        # or through app settings comes from `init_app` invoking
        self.session = self.create_scoped_session()

        self.Model = self.make_declarative_base()

        # config
        if app:
            assert isinstance(app, TorextApp), 'app should be an instance of TorextApp'
            self.config = self.init_config(app.settings['SQLALCHEMY'])
        else:
            self.config = self.init_config(config)
        if uri:
            self.config['uri'] = uri

        if self.config['uri']:
            self.session.configure(bind=self.engine)

        self.app = app
        self._include_sqlalchemy()

    def init_app(self, app):
        self.app = app
        self.config = self.init_config(app.settings['SQLALCHEMY'])
        self.session.configure(bind=self.engine)

    def create_scoped_session(self, options=None):
        return scoped_session(sessionmaker(query_cls=BaseQuery))

    def make_declarative_base(self):
        base = declarative_base(cls=_Model, name='Model')
        base.query = self.session.query_property()
        return base

    def _include_sqlalchemy(self):
        for module in sqlalchemy, sqlalchemy.orm:
            for key in module.__all__:
                if not hasattr(self, key):
                    setattr(self, key, getattr(module, key))
        # Note: self.Table does not attempt to be a SQLAlchemy Table class.
        self.Table = _make_table(self)
        self.relationship = _wrap_with_default_query_class(self.relationship)
        self.relation = _wrap_with_default_query_class(self.relation)
        self.dynamic_loader = _wrap_with_default_query_class(self.dynamic_loader)

    def init_config(self, incoming=None):
        config = {
            'uri': None,
            'pool_size': None,
            'pool_timeout': None,
            'pool_recycle': None,
            'echo': False,
        }
        if incoming:
            for k in config:
                if k in incoming:
                    config[k] = incoming[k]
        return config

    @property
    def engine(self):
        if not self._engine:
            self._engine = self.get_engine()
        return self._engine

    def get_engine(self):
        assert self.config['uri'], 'uri should not be empty,'\
            ' assign by call init_app or explicitly passing'
        uri_obj = make_url(self.config['uri'])
        options = {
            'convert_unicode': True,
            'echo': self.config.get('echo', False)
        }
        for k in ('pool_size', 'pool_timeout', 'pool_recycle'):
            v = self.config.get(k, None)
            if v is not None:
                options[k] = v
        self._apply_driver_hacks(uri_obj, options)
        logging.info('SQLAlchemy engine config: %s, %s', uri_obj, options)
        engine = sqlalchemy.create_engine(uri_obj, **options)
        return engine

    def _apply_driver_hacks(self, uri_obj, options):
        if uri_obj.drivername == 'mysql':
            uri_obj.query.setdefault('charset', 'utf8')
            options.setdefault('pool_size', 10)
            options.setdefault('pool_recycle', 7200)
        elif uri_obj.drivername == 'sqlite':
            pool_size = options.get('pool_size')
            in_memory = False
            # we go to memory and the pool size was explicitly set to 0
            # which is fail.  Let the user know that
            if uri_obj.database in (None, '', ':memory:'):
                in_memory = True
                if pool_size == 0:
                    raise RuntimeError('SQLite in memory database with an '
                                       'empty queue not possible due to data '
                                       'loss.')
            # if pool size is None or explicitly set to 0 we assume the
            # user did not want a queue for this sqlite connection and
            # hook in the null pool.
            elif not pool_size:
                from sqlalchemy.pool import NullPool
                options['poolclass'] = NullPool

            # if it's not an in memory database we make the path absolute.
            if not in_memory and self.app:
                uri_obj.database = os.path.join(self.app.root_path, uri_obj.database)

    def _execute_operation(self, operation):
            op = getattr(self.Model.metadata, operation)
            op(self.engine)

    def create_all(self):
        self._execute_operation('create_all')

    def drop_all(self):
        self._execute_operation('drop_all')

    def reflect(self):
        self._execute_operation('reflect')

    def __repr__(self):
        return '<%s engine=%r>' % (
            self.__class__.__name__,
            self.config.get('uri')
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    import sqlalchemy
except ImportError:
    print 'sqlalchemy is not installed, skip testing'
    from nose.plugins.skip import SkipTest
    raise SkipTest

import unittest
from torext.app import TorextApp
from torext.sql import SQLAlchemy


class MysqlTestCase(unittest.TestCase):
    def setUp(self):
        app = TorextApp()
        app.settings['DEBUG'] = False
        app.settings['SQLALCHEMY'] = {
            'uri': 'mysql://root:zxsaqw21@localhost/torext_test',
            'echo': True
        }
        app.setup()
        db = SQLAlchemy(app=app)

        class User(db.Model):
            id = db.Column(db.Integer, primary_key=True)
            name = db.Column(db.String(20))

        self.User = User

        db.create_all()

        self.app = app
        self.db = db

    def tearDown(self):
        # clean possible opened transactions
        self.db.session.commit()

        self.db.drop_all()

    def test_create_delete(self):
        u = self.User(name='reorx')
        self.db.session.add(u)
        self.db.session.commit()

        us = self.User.query.all()
        assert len(us) == 1

        self.db.session.delete(u)

        us = self.User.query.all()
        assert len(us) == 0


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import copy
#import pkgutil
#import logging
import datetime
import functools


try:
    import simplejson as pyjson
except ImportError:
    import json as pyjson

try:
    from tornado.util import raise_exc_info
except ImportError:
    def raise_exc_info(exc_info):
        """Re-raise an exception (with original traceback) from an exc_info tuple.

        The argument is a ``(type, value, traceback)`` tuple as returned by
        `sys.exc_info`.
        """
        # 2to3 isn't smart enough to convert three-argument raise
        # statements correctly in some cases.
        if isinstance(exc_info[1], exc_info[0]):
            raise exc_info[1], None, exc_info[2]
            # After 2to3: raise exc_info[1].with_traceback(exc_info[2])
        else:
            # I think this branch is only taken for string exceptions,
            # which were removed in Python 2.6.
            raise exc_info[0], exc_info[1], exc_info[2]
            # After 2to3: raise exc_info[0](exc_info[1]).with_traceback(exc_info[2])


def generate_cookie_secret():
    import uuid
    import base64
    return base64.b64encode(uuid.uuid4().bytes + uuid.uuid4().bytes)


json_decode = functools.partial(pyjson.loads, encoding='utf-8')


json_encode = functools.partial(pyjson.dumps, ensure_ascii=False)


#def force_int(value, desire=0, limit=100):
    #try:
        #value = int(value)
    #except:
        #value = desire
    #if value > limit:
        #return limit / 2
    #return value


def timesince(t):
    if not isinstance(t, datetime.datetime):
        raise TypeError('Time should be instance of datetime.datetime')
    now = datetime.datetime.utcnow()
    delta = now - t
    if not delta.days:
        if delta.seconds / 3600:
            return '{0} hours ago'.format(delta.seconds / 3600)
        return '{0} minutes ago'.format(delta.seconds / 60)
    if delta.days / 365:
        return '{0} years ago'.format(delta.days / 365)
    if delta.days / 30:
        return '{0} months ago'.format(delta.days / 30)
    return '{0} days ago'.format(delta.days)


def pprint(o):
    import pprint as PPrint
    pprinter = PPrint.PrettyPrinter(indent=4)
    pprinter.pprint(o)


class SingletonMixin(object):
    """Globally hold one instance class

    Usage:
    >>> class SpecObject(SingletonMixin):
    ...     pass

    >>> ins = SpecObject.instance()
    """
    @classmethod
    def instance(cls, *args, **kwgs):
        """Will be the only instance"""
        if not hasattr(cls, "_instance"):
            cls._instance = cls(*args, **kwgs)
        return cls._instance


def split_kwargs(kwgs_tuple, kwgs):
    _kwgs = {}
    for i in kwgs_tuple:
        if i in kwgs:
            _kwgs[i] = kwgs.pop(i)
    return _kwgs


class ObjectDict(dict):
    """
    retrieve value of dict in dot style
    """
    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError('Has no attribute %s' % key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError:
            raise AttributeError(key)

    def __str__(self):
        return '<ObjectDict %s >' % dict(self)


try:
    from collections import OrderedDict
except ImportError:
    class OrderedDict(dict):
        """
        A dictionary that keeps its keys in the order in which they're inserted.

        # No initial order
        >>> d = OrderedDict({'a': 1, 'b': 2})

        # Ordered
        >>> d = OrderedDict([('a', 1), ('b', 2)])

        # Ordered
        >>> d = OrderedDict()
        >>> d['a'] = 1
        >>> d['b'] = 2

        # Chaos-ordered
        >>> d = OrderedDict({'a': 1, 'b': 2})
        >>> d['c'] = 3
        >>> d['d'] = 4
        """
        def __new__(cls, *args, **kwargs):
            instance = super(OrderedDict, cls).__new__(cls, *args, **kwargs)
            instance.ordered_keys = []
            return instance

        def __init__(self, data=None):
            if data is None or isinstance(data, dict):
                data = data or []
                super(OrderedDict, self).__init__(data)
                self.ordered_keys = list(data) if data else []
            else:
                super(OrderedDict, self).__init__()
                super_set = super(OrderedDict, self).__setitem__
                for key, value in data:
                    # Take the ordering from first key
                    if key not in self:
                        self.ordered_keys.append(key)
                    # But override with last value in data (dict() does this)
                    super_set(key, value)

        def __deepcopy__(self, memo):
            return self.__class__([(key, copy.deepcopy(value, memo))
                                   for key, value in self.items()])

        def __copy__(self):
            # The Python's default copy implementation will alter the state
            # of self. The reason for this seems complex but is likely related to
            # subclassing dict.
            return self.copy()

        def __setitem__(self, key, value):
            if key not in self:
                self.ordered_keys.append(key)
            super(OrderedDict, self).__setitem__(key, value)

        def __delitem__(self, key):
            super(OrderedDict, self).__delitem__(key)
            self.ordered_keys.remove(key)

        def __iter__(self):
            return iter(self.ordered_keys)

        def __reversed__(self):
            return reversed(self.ordered_keys)

        def pop(self, k, *args):
            result = super(OrderedDict, self).pop(k, *args)
            try:
                self.ordered_keys.remove(k)
            except ValueError:
                # Key wasn't in the dictionary in the first place. No problem.
                pass
            return result

        def popitem(self):
            result = super(OrderedDict, self).popitem()
            self.ordered_keys.remove(result[0])
            return result

        def iteritems(self):
            for key in self.ordered_keys:
                yield key, self[key]

        def iterkeys(self):
            for key in self.ordered_keys:
                yield key

        def itervalues(self):
            for key in self.ordered_keys:
                yield self[key]

        def items(self):
            return list(self.iteritems())

        def keys(self):
            return self.ordered_keys[:]

        def values(self):
            return list(self.itervalues())

        def update(self, dict_):
            for k, v in dict_.iteritems():
                self[k] = v

        def setdefault(self, key, default):
            if key in self:
                return self[key]
            else:
                self[key] = default
                return default

        def copy(self):
            """Returns a copy of this object."""
            # This way of initializing the copy means it works for subclasses, too.
            return self.__class__(self)

        def __repr__(self):
            """
            Replaces the normal dict.__repr__ with a version that returns the keys
            in their sorted order.
            """
            return '{%s}' % ', '.join(['%r: %r' % (k, v) for k, v in self.iteritems()])

        def clear(self):
            super(OrderedDict, self).clear()
            self.ordered_keys = []


#def import_underpath_module(path, name):
    #"""
    #arguments::
    #:name :: note that name do not contain `.py` at the end
    #"""
    #importer = pkgutil.get_importer(path)
    #logging.debug('loading handler module: ' + name)
    #return importer.find_module(name).load_module(name)


#def autoload_submodules(dirpath):
    #"""Load submodules by dirpath
    #NOTE. ignore packages
    #"""
    #import pkgutil
    #importer = pkgutil.get_importer(dirpath)
    #return (importer.find_module(name).load_module(name)
            #for name, is_pkg in importer.iter_modules())


######################################
# borrow from django.utils.importlib #
######################################

# Taken from Python 2.7 with permission from/by the original author.

def _resolve_name(name, package, level):
    """Return the absolute name of the module to be imported."""
    if not hasattr(package, 'rindex'):
        raise ValueError("'package' not set to a string")
    dot = len(package)
    for x in xrange(level, 1, -1):
        try:
            dot = package.rindex('.', 0, dot)
        except ValueError:
            raise ValueError("attempted relative import beyond top-level\
                package")
    return "%s.%s" % (package[:dot], name)


def import_module(name, package=None):
    """Import a module.

    The 'package' argument is required when performing a relative import. It
    specifies the package to use as the anchor point from which to resolve the
    relative import to an absolute import.

    """
    if name.startswith('.'):
        if not package:
            raise TypeError("relative imports require the 'package' argument")
        level = 0
        for character in name:
            if character != '.':
                break
            level += 1
        name = _resolve_name(name[level:], package, level)
    __import__(name)
    return sys.modules[name]


_abspath = lambda x: os.path.abspath(x)
_join = lambda x, y: os.path.join(x, y)


def add_to_syspath(pth, relative_to=None):
    if relative_to:
        pth = _join(relative_to, pth)
    if _abspath(pth) in [_abspath(i) for i in sys.path]:
        print 'path %s is in sys.path, pass' % pth
    else:
        print 'add path %s to sys.path' % pth
        sys.path.insert(0, pth)


def start_shell(extra_vars=None):
    import code
    import rlcompleter
    import readline

    class irlcompleter(rlcompleter.Completer):
        def complete(self, text, state):
            if text == "":
                #you could  replace \t to 4 or 8 spaces if you prefer indent via spaces
                return ['    ', None][state]
            else:
                return rlcompleter.Completer.complete(self, text, state)

    readline.parse_and_bind("tab: complete")
    readline.set_completer(irlcompleter().complete)

    import __main__
    if extra_vars:
        __main__.__dict__.update(
            dict((k, v) for k, v in extra_vars.iteritems() if not k.startswith('__')))

    # As completer class search complement variables in `__main__.__dict__`
    # (`self.namespace = __main__.__dict__` in 'rlcompleter.Completer.complete'),
    # and the code is executed by `exec code in self.locals`
    # (in `code.InteractiveInterpreter.runcode`),
    # `__main__.__dict__` must be used as the local variables scope,
    # not `globals()`, `locals()` or any other dict,
    # (`__main__` module is explained here: http://docs.python.org/2/library/__main__.html)
    shell = code.InteractiveConsole(__main__.__dict__)
    shell.interact()


def start_shell_a(extra_vars=None):

    import code
    import rlcompleter
    import readline

    #class irlcompleter(rlcompleter.Completer):
        #def complete(self, text, state):
            #if text == "":
                ##you could  replace \t to 4 or 8 spaces if you prefer indent via spaces
                #return ['    ', None][state]
            #else:
                #return rlcompleter.Completer.complete(self, text, state)

    #readline.set_completer(irlcompleter().complete)
    readline.parse_and_bind("tab: complete")

    #_globals = globals()
    #_globals.update(extra_vars)
    #import __builtin__ as bt
    #local_vars = {
        #'__name__': '__main__',
        #'__package__': None,
        #'__doc__': None,
        #'__builtins__': bt
    #}
    g = globals()
    g['__name__'] == '__main__'
    print g['datetime']
    import __main__
    #shell = code.InteractiveConsole(g)
    #shell.interact()
    code.interact(local=__main__.__dict__)


def fix_request_arguments(arguments):
    return dict((k, v[0]) for k, v in arguments.iteritems())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import json
from nose.tools import assert_raises
from torext import params
from torext.errors import ValidationError, ParamsInvalidError


def test_regex():
    pairs = [
        (r'^\w+', 'hell*', True),
        (r'^\w+', '*ello', False),

        (r'\w+$', 'hell*', False),
        (r'\w+$', '*ello', True),

        (r'^\w+$', 'hello', True),
        (r'^\w+$', '*ello', False),
        (r'^\w+$', 'hell*', False)
    ]

    for pattern, match, result in pairs:
        yield check_regex, pattern, match, result


def check_regex(pattern, match, result):
    field = params.RegexField(pattern=pattern)
    if result:
        field.validate(match)
    else:
        with assert_raises(ValidationError):
            field.validate(match)


def test_words():
    f0 = params.WordField()
    f0.validate('')
    f0.validate('goodstr')
    with assert_raises(ValidationError):
        f0.validate('should not contain space')
    with assert_raises(ValidationError):
        f0.validate('andother*(*^&')

    f1 = params.WordField(length=(4, 8))
    f1.validate('asdf')
    f1.validate('asdfasdf')
    with assert_raises(ValidationError):
        f1.validate('s')
    with assert_raises(ValidationError):
        f1.validate('longggggg')

    f2 = params.WordField(null=False)
    with assert_raises(ValidationError):
        f2.validate('')


def test_email():
    pairs = [
        ('i@t.cn', True),
        ('longname@longdomain.cn', True),
        ('nor@mal.thr', True),
        ('nor@mal.four', True),
        ('nor@mal.fivee', True),
        ('nor@mal.sixxxx', True),
        ('nor@mal.sevennnn', False),
        ('nor@mal', False),
        ('@mal.com', False),
    ]
    for email, result in pairs:
        yield check_email, email, result


def check_email(email, result):
    f = params.EmailField()
    if result:
        f.validate(email)
    else:
        assert_raises(ValidationError, f.validate, email)
        pass


def test_url():
    pairs = [
        ('http://hello.com', True),
        ('https://askdjfasdf.asdfasdf.com/', True),
        ('ftp://www.google.com', True),
        ('ssh://www.google.com', False),
        ('http://have.punc*tu*rat@ions.com', False),
        ('http://a.b.c.d.e.f.g.com', True),
        ('http://t.cn/@#$#$(*&', True),
    ]
    for url, result in pairs:
        yield check_url, url, result


def check_url(url, res):
    f = params.URLField()
    if res:
        f.validate(url)
    else:
        assert_raises(ValidationError, f.validate, url)


def test_intstring():
    pairs = [
        ('a', False),
        ('0b', False),
        ('1', True),
        ('2', False, {'min': 3}),
        ('100', False, {'max': 99}),
        ('1023', False, {'min': 1024, 'max': 1024}),
        ('1024', True, {'min': 1024, 'max': 1024}),
        ('1025', False, {'min': 1024, 'max': 1024}),
    ]
    for args in pairs:
        yield check_intstring, args


def check_intstring(args):
    kwgs = {}
    if len(args) > 2:
        kwgs = args[2]
    f = params.IntegerField(**kwgs)
    if args[1]:
        f.validate(args[0])
    else:
        assert_raises(ValidationError, f.validate, args[0])


def test_uuidstring():
    pairs = [
        ('asdf', False),
        ('1234', False),
        ('216edfae-19c0-11e3-9e93-10604b8a89ab', True)
    ]

    for s, res in pairs:
        yield check_uuidstring, s, res


def check_uuidstring(s, res):
    f = params.UUIDField()
    if res:
        f.validate(s)
    else:
        with assert_raises(ValidationError):
            f.validate(s)


def test_simple_list():
    list_field = params.ListField(choices=['a', 'b', 'c'])

    list_field.validate(['a'])
    list_field.validate(['a', 'b', 'c'])

    with assert_raises(ValidationError):
        list_field.validate(['b', 'c', 'd'])
    with assert_raises(ValidationError):
        list_field.validate(['z', 'a', 'b'])


def test_type_list():
    list_field = params.ListField(item_field=params.IntegerField(min=1, max=9), choices=[1, 2, 3])

    list_field.validate(['1', '2', '3'])
    with assert_raises(ValidationError):
        list_field.validate(['0', '1', '2'])
    with assert_raises(ValidationError):
        list_field.validate(['1', '2', '3', '4'])
    with assert_raises(ValidationError):
        list_field.validate(['a', '2', '3'])


class FakeParams(params.ParamSet):
    id = params.IntegerField('wat are you?', required=True, min=0)
    name = params.WordField('name should be a 1~8 length string, and is required', required=True, length=(1, 8))
    email = params.EmailField('email should be a valid email format, and is required', required=True)
    content = params.Field('content should be a 1~20 length string', length=(1, 20))


def test_param():
    data_pairs = [
        ({
            'id': '0',
            'name': 'asuka',
            'email': 'asuka@nerv.com'
        }, 0),
        ({
            'id': '2',
            'name': 'lilith',
            'email': 'l@eva.com',
            'content': 'with adon'
        }, 0),
        ({
            'id': 'a3',
            'name': 'ayanami',
            'email': 'rei@nerv.com'
        }, 1),
        ({
            'id': 'b3',
            'name': 'shinjigivebackmyayanami',
            'email': 'yikali@nerv.com'
        }, 2),
        ({
            'id': 'c4',
            'name': 'E V A',
            'email': 'eva@god',
            'content': 'Gainax launched a project to create a movie ending for the series in 1997. The company first released Death and Rebirth on March 15'
        }, 4),
        ({}, 3),
        ({
            'id': 'd5'
        }, 3),
        ({
            'id': '999'
        }, 2),
        ({
            'content': 'The project to complete the final episodes (retelling episodes 25 and 26 of the series) was completed later in 1997 and released on July 19 as The End of Evangelion. '
        }, 4)
    ]

    for data, error_num in data_pairs:
        yield check_param, data, error_num


def check_param(data, error_num):
    params = FakeParams(**data)
    print error_num, len(params.errors), params.errors
    assert error_num == len(params.errors)


PARAMS_ID_MSG = 'id shoud be int larger than 1'
PARAMS_TOKEN_MSG = 'token should be a 32 length string'
PARAMS_TAG_MSG = 'tag should be word without punctuations, less than 8 characters'
PARAMS_FROM = 'from should be less than 8 characters'


class WebTestCase(unittest.TestCase):
    def setUp(self):
        from torext.app import TorextApp
        from torext.handlers import BaseHandler

        app = TorextApp()

        class APIParams(params.ParamSet):
            id = params.IntegerField(PARAMS_ID_MSG, required=True, min=1)
            token = params.Field(PARAMS_TOKEN_MSG, required=True, length=32)

            tag = params.WordField(PARAMS_TAG_MSG, length=8, default='foo')
            from_ = params.WordField(PARAMS_FROM, key='from', required=False, length=16)
            text_anyway = params.WordField()
            text_not_null = params.WordField(null=False)

        @app.route('/api')
        class APIHandler(BaseHandler):
            @APIParams.validation_required
            def get(self):
                print 'arguments', self.request.arguments
                print 'params', self.params
                self.write_json(self.params.to_dict(include_none=True))
                pass

            def post(self):
                self.write('ok')
                pass

        # let exceptions raised in handler be rethrowed in test function
        self.c = app.test_client(raise_handler_exc=True)

    def tearDown(self):
        self.c.close()

    def test_good(self):
        print 'test good'
        resp = self.c.get('/api', {
            'id': 1,
            'token': '0cc175b9c0f1b6a831c399e269772661'
        })
        assert resp.code == 200
        data = json.loads(resp.body)
        assert data['id'] == 1
        assert data['token'] == '0cc175b9c0f1b6a831c399e269772661'
        assert data['tag'] == 'foo'
        assert data['from'] is None

    def test_bad_id(self):
        resp = self.c.get('/api', {
            'id': 'a2',
            'token': '0cc175b9c0f1b6a831c399e269772661'
        })
        assert resp.code == 500
        self.assertRaises(ParamsInvalidError, self.c.get_handler_exc)

    def test_bad_token(self):
        resp = self.c.get('/api', {
            'id': '1',
            'token': '1c399e26977266'
        })
        assert resp.code == 500
        self.assertRaises(ParamsInvalidError, self.c.get_handler_exc)

    def test_bad_tag(self):
        resp = self.c.get('/api', {
            'id': '1',
            'token': '0cc175b9c0f1b6a831c399e269772661',
            'tag': 'good man'
        })
        assert resp.code == 500
        self.assertRaises(ParamsInvalidError, self.c.get_handler_exc)

    def test_null(self):
        resp = self.c.get('/api', {
            'id': 1,
            'token': '0cc175b9c0f1b6a831c399e269772661',
            'text_anyway': ''
        })
        assert resp.code == 200
        data = json.loads(resp.body)
        print 'data', data
        assert data['text_anyway'] == ''

        resp = self.c.get('/api', {
            'id': 1,
            'token': '0cc175b9c0f1b6a831c399e269772661',
            'text_anyway': '',
            'text_not_null': ''
        })
        assert resp.code == 500


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Always stay on 0.9 before documentation is finished
__version__ = '0.9.4-r2'

from setuptools import setup

with open('README.rst', 'r') as f:
    description = f.read()

setup(
    license='License :: OSI Approved :: MIT License',
    name='torext',
    version=__version__,
    author='reorx',
    author_email='novoreorx@gmail.com',
    url='http://github.com/reorx/torext',
    description="The missing tornado mate",
    long_description=description,
    packages=[
        'torext',
        'torext.handlers',
        'torext.scripts',
        'torext.test',
    ],
    package_data={
        'torext': [
            'templates/project/__init__.py',
            'templates/project/app.py',
            'templates/project/settings.py',
        ]
    },
    entry_points={
        'console_scripts': [
            'torext-sketch = torext.scripts.sketch:main',
        ]
    },
    install_requires=[
        'tornado>=3.0'
    ],
)



PROJECT = '{project_name}'

LOCALE = 'en_US'

PROCESSES = 1

PORT = 8000

DEBUG = True

LOGGERS = {
    '': {
        'level': 'INFO',
        'fmt': '%(color)s[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s',
        #'fmt': '[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d] %(message)s',
        'datefmt': '%Y-%m-%d %H:%M:%S',
    }
}

LOG_REQUEST = True

LOG_RESPONSE = False

TIME_ZONE = 'Asia/Shanghai'

STATIC_PATH = 'static'

TEMPLATE_PATH = 'template'

# You can use jinja2 instead
TEMPLATE_ENGINE = 'tornado'

LOGGING_IGNORE_URLS = [
    '/favicon.ico',
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from torext import utils


def test_generate_cookie_secret():
    assert utils.generate_cookie_secret()


def test_json_decode():
    assert isinstance(utils.json_decode('{"a": 1}'), dict)


def test_json_encode():
    assert isinstance(utils.json_encode({"a": 1}), str)


def test_timesince():
    import datetime
    assert utils.timesince(datetime.datetime.now())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import time
import copy
import socket
import logging

from tornado.ioloop import IOLoop
from tornado.httpserver import HTTPServer
from tornado.web import Application

from torext import settings
from torext import errors
from torext.testing import TestClient, AppTestCase
from torext.log import set_nose_formatter, app_log, request_log
from torext.route import Router
from torext.utils import json_encode, json_decode


class TorextApp(object):
    """
    Simplify the way to setup and run an app instance
    """
    current_app = None

    def __init__(self, settings_module=None, extra_settings=None,
                 application_options=None, httpserver_options=None,
                 io_loop=None):
        """
        Automatically involves torext's settings

        related keys:
            DEBUG
            TEMPLATE_PATH
            STATIC_PATH
            STATIC_URL_PREFIX
            STATIC_HANDLER_CLASS
            STATIC_HANDLER_ARGS
            COOKIE_SECRET
            XSRF_COOKIES
            UI_MODULES
            UI_METHODS
            GZIP
            XHEADERS
        """
        global settings

        if settings_module:
            self.module_config(settings_module)
        if extra_settings:
            self.update_settings(extra_settings)
        self._application_options = application_options
        self._httpserver_options = httpserver_options
        self.io_loop = io_loop
        self.is_setuped = False
        self.handlers = []
        self.default_host = '.*$'
        self.host_handlers = {
            self.default_host: []
        }
        self.application = None
        self.project = None
        self.uimodules = {}
        self.json_encoder = json_encode
        self.json_decoder = json_decode

        self.settings = settings

        TorextApp.current_app = self

    def update_settings(self, incoming, convert_type=False, log_changes=False):
        global settings

        def _log(s):
            if log_changes:
                print s

        for i in incoming:
            incoming_v = incoming[i]

            if i in settings:
                _log('Settings update "%s": %s -> %s' % (i, settings[i], incoming_v))

                if convert_type:
                    incoming_v = _convert_type(incoming_v, settings[i])
            else:
                _log('Settings add "%s": %s' % (i, incoming_v))

            settings[i] = incoming_v

        self.is_setuped = False

    def set_root_path(self, root_path=None, settings_module=None):
        """
        root_path will only be used for 3 situations:
        1. fix ``static_path`` and ``template_path``
        2. check project's directory name with the value in settings.PROJECT
        3. append parent path to sys.path
        """
        if root_path:
            self.root_path = root_path
            return

        if settings_module:
            self.root_path = os.path.dirname(os.path.abspath(settings_module.__file__))
            return

        # try to guess which module import app.py
        import inspect

        caller = inspect.stack()[1]
        caller_module = inspect.getmodule(caller[0])
        assert hasattr(caller_module, '__file__'), 'Caller module %s should have __file__ attr' % caller_module
        self.root_path = os.path.dirname(os.path.abspath(caller_module.__file__))

    def get_httpserver_options(self):
        keys = ('xheaders', )
        options = {}
        for k in keys:
            k_upper = k.upper()
            if k_upper in settings:
                options[k] = settings[k_upper]

        if self._httpserver_options:
            for k, v in self._httpserver_options.iteritems():
                options[k] = v

        return options

    def get_application_options(self):
        # TODO full list options
        options_keys = [
            'debug',
            'static_path',
            'template_path',
            'cookie_secret',
            'log_function',
            'ui_modules',
            'static_handler_class',
            'static_handler_args',

            'autoreload',
            'compiled_template_cache',
            'static_hash_cache',
            'serve_traceback'
        ]
        options = {
            'log_function': self._log_function,
            'ui_modules': self.uimodules,
        }

        for k in options_keys:
            k_upper = k.upper()
            if k_upper in settings:
                options[k] = settings[k_upper]

        if hasattr(self, 'root_path'):
            self._fix_paths(options)

        if self._application_options:
            for k, v in self._application_options.iteritems():
                options[k] = v

        return options

    def _fix_paths(self, options):
        """
        fix `static_path` and `template_path` to be absolute
        path according to self.root_path so that PWD can be ignoreed.
        """
        for k in ('template_path', 'static_path'):
            if k in options:
                v = options.pop(k)
                if not os.path.isabs(v):
                    v = os.path.abspath(
                        os.path.join(self.root_path, v))
                    app_log.debug('Fix %s to be absolute: %s' % (k, v))
                options[k] = v

    def _get_handlers_on_host(self, host=None):
        if not host:
            handlers = self.host_handlers[self.default_host]
        else:
            self.host_handlers.setdefault(host, [])
            handlers = self.host_handlers[host]
        return handlers

    def route(self, url, host=None):
        """This is a decorator
        """
        def fn(handler_cls):
            handlers = self._get_handlers_on_host(host)
            handlers.insert(0, (url, handler_cls))
            return handler_cls
        return fn

    def route_many(self, *args, **kwargs):
        """
        >>> from torext.route import include
        >>> app = TorextApp()
        >>> app.route_many([
        ...     ('/account', include('account.views')),
        ...     ('/account', include('account.views')),
        ... ], '^account.example.com$')
        """
        if len(args) == 2:
            prefix, rules = args
        elif len(args) == 1:
            prefix = None
            rules = args[0]
        else:
            raise ValueError('The amount of args of route_many method can only be one or two')
        router = Router(rules, prefix=prefix)
        self.add_handlers(router.get_handlers(), host=kwargs.get('host'))

    def add_handlers(self, handlers, host=None):
        handlers_container = self._get_handlers_on_host(host)
        handlers_container[0:0] = handlers

    def module_config(self, settings_module):
        """
        Optional function
        """
        assert hasattr(settings_module, '__file__'), 'settings must be a module'
        # set root_path according to module file
        self.set_root_path(settings_module=settings_module)
        app_log.debug('Set root_path: %s', self.root_path)

        global settings

        self.update_settings(dict(
            [(i, getattr(settings_module, i)) for i in dir(settings_module)
             if not i.startswith('_') and i == i.upper()]))

        settings._module = settings_module

        # keep a mapping to app on settings object
        settings._app = self

    def command_line_config(self):
        """
        settings.py is the basis

        if wants to change them by command line arguments,
        the existing option will be transformed to the value type in settings.py
        the unexisting option will be treated as string by default,
        and transform to certain type if `!<type>` was added after the value.

        example:
        $ python app.py --PORT=1000

        NOTE This method is deprecated, use `torext.script` to parse command line arguments instead.
        """

        args = sys.argv[1:]
        args_dict = {}
        existed_keys = []
        new_keys = []

        for t in args:
            if not t.startswith('--'):
                raise errors.ArgsParseError('Bad arg: %s' % t)
            try:
                key, value = tuple(t[2:].split('='))
            except:
                raise errors.ArgsParseError('Bad arg: %s' % t)

            args_dict[key] = value

            if key in settings:
                existed_keys.append(key)
            else:
                new_keys.append(key)

        if existed_keys:
            app_log.debug('Changed settings:')
            for i in existed_keys:
                before = settings[i]
                type_ = type(before)
                if type_ is bool:
                    if args_dict[i] == 'True':
                        _value = True
                    elif args_dict[i] == 'False':
                        _value = False
                    else:
                        raise errors.ArgsParseError('%s should only be True or False' % i)
                else:
                    _value = type_(args_dict[i])
                settings[i] = _value
                app_log.debug('  %s  [%s]%s (%s)', i, type(settings[i]), settings[i], before)

        if new_keys:
            app_log.debug('New settings:')
            for i in new_keys:
                settings[i] = args_dict[i]
                app_log.debug('  %s  %s', i, args_dict[i])

        # NOTE if ``command_line_config`` is called, logging must be re-configed
        self.update_settings({})

    def setup(self):
        """This function will be called both before `run` and testing started.
        """
        testing = settings.get('TESTING')

        if testing:
            # Fix nose handler in testing situation.
            config = settings['LOGGERS'].get('', {})
            set_nose_formatter(config)
            print 'testing, set nose formatter: %s' % config

        # reset timezone
        os.environ['TZ'] = settings['TIME_ZONE']
        time.tzset()

        # determine project name
        if settings._module:
            project = os.path.split(self.root_path)[1]
            if settings['PROJECT']:
                assert settings['PROJECT'] == project, 'PROJECT specialized in settings (%s) '\
                    'should be the same as project directory name (%s)' % (settings['PROJECT'], project)
            else:
                settings['PROJECT'] = project

        # PROJECT should be importable as a python module
        if settings['PROJECT']:
            # add upper directory path to sys.path if not in
            if settings._module:
                _abs = os.path.abspath
                parent_path = os.path.dirname(self.root_path)
                if not _abs(parent_path) in [_abs(i) for i in sys.path]:
                    sys.path.insert(0, parent_path)
                    app_log.info('Add %s to sys.path' % _abs(parent_path))
            try:
                __import__(settings['PROJECT'])
                app_log.debug('import package `%s` success' % settings['PROJECT'])
            except ImportError:
                raise ImportError('PROJECT could not be imported, may be app.py is outside the project'
                                  'or there is no __init__ in the package.')

        self.is_setuped = True

    def _init_infrastructures(self):
        multiprocessing = False
        if settings['PROCESSES'] and settings['PROCESSES'] > 1:
            if settings['DEBUG']:
                app_log.info('Multiprocess could not be used in debug mode')
            else:
                multiprocessing = True

        if self.io_loop:
            if not self.io_loop.initialized():
                # this means self.io_loop is a customized io_loop, so `install` should be called
                # to make it the singleton instance
                #print self.io_loop.__class__.__name__
                self.io_loop.install()
        else:
            # NOTE To support running tornado for multiple processes, we do not instance ioloop if multiprocessing is True
            if not multiprocessing:
                self.io_loop = IOLoop.instance()

        application = self._make_application()

        http_server_options = self.get_httpserver_options()
        http_server = HTTPServer(application, io_loop=self.io_loop, **http_server_options)
        listen_kwargs = {}
        if settings.get('ADDRESS'):
            listen_kwargs['address'] = settings.get('ADDRESS')

        if multiprocessing:
            # Multiprocessing mode
            try:
                http_server.bind(settings['PORT'], **listen_kwargs)
            except socket.error, e:
                app_log.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
                raise e
            http_server.start(settings['PROCESSES'])
        else:
            # Single process mode
            try:
                http_server.listen(settings['PORT'], **listen_kwargs)
            except socket.error, e:
                app_log.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
                raise e

        self.http_server = http_server
        self.application = application

    @property
    def is_running(self):
        if self.io_loop:
            return self.io_loop._running
        return False

    def _instance_ioloop(self):
        if not self.io_loop:
            self.io_loop = IOLoop.instance()

    def run(self):
        if not self.is_setuped:
            self.setup()

        self._init_infrastructures()

        if not settings.get('TESTING'):
            self.log_app_info()

        try:
            self._instance_ioloop()
            self.io_loop.start()
        except KeyboardInterrupt:
            print '\nStopping ioloop.. ',
            IOLoop.instance().stop()
            print 'Exit'
            sys.exit(0)

    def log_app_info(self):
        mode = settings['DEBUG'] and 'Debug' or 'Product'
        content = '\nMode %s, Service Info:' % mode
        loggers_info = {}
        for k in settings['LOGGERS']:
            _logger = logging.getLogger(k)
            #loggers_info[k] = {i: getattr(_logger, i) for i in ('level', 'handlers', 'propagate')}
            loggers_info[k] = dict((i, getattr(_logger, i)) for i in ('level', 'handlers', 'propagate'))
            level = loggers_info[k]['level']
            loggers_info[k]['level'] = '%s (%s)' % (level, logging._levelNames[level])

        info = {
            'Project': settings['PROJECT'] or 'None (better be assigned)',
            'Port': settings['PORT'],
            'Processes': settings['DEBUG'] and 1 or settings['PROCESSES'],
            'Loggers': loggers_info,
            'Locale': settings['LOCALE'],
            'Debug': settings['DEBUG'],
            'Home': 'http://127.0.0.1:%s' % settings['PORT'],
        }

        #if settings['DEBUG']:
        buf = []
        for host, rules in self.application.handlers:
            buf.append(host.pattern)
            for i in rules:
                buf.append('  ' + i.regex.pattern)
        info['URL Patterns(by sequence)'] = '\n    ' + '\n    '.join(buf)

        for k in ['Project', 'Port', 'Processes',
                  'Loggers', 'Locale', 'Debug', 'Home', 'URL Patterns(by sequence)']:
            content += '\n- %s: %s' % (k, info[k])

        app_log.info(content)

    def test_client(self, **kwgs):
        return TestClient(self, **kwgs)

    @property
    def TestCase(_self):
        class CurrentTestCase(AppTestCase):
            def get_client(self):
                return _self.test_client()
        return CurrentTestCase

    def register_uimodules(self, **kwargs):
        self.uimodules.update(kwargs)

    def register_json_encoder(self, encoder_func):
        self.json_encoder = encoder_func
        return encoder_func

    def register_json_decoder(self, decoder_func):
        self.json_decoder = decoder_func
        return decoder_func

    def register_application_configurator(self, config_func):
        self.application_configurator = config_func
        return config_func

    def _make_application(self, application_class=Application):
        options = self.get_application_options()
        app_log.debug('%s settings: %s', application_class.__name__, options)

        # this method intended to be able to called for multiple times,
        # so attributes should not be changed, just make a copy
        host_handlers = copy.copy(self.host_handlers)
        top_host_handlers = host_handlers.pop('.*$')
        application = application_class(top_host_handlers, **options)

        if host_handlers:
            for host, handlers in host_handlers.iteritems():
                application.add_handlers(host, handlers)

        # call `application_configurator` to do extra setups
        self.application_configurator(application)
        return application

    def application_configurator(self, *args, **kwargs):
        pass

    def wsgi_application(self):
        from tornado.wsgi import WSGIApplication
        return self._make_application(application_class=WSGIApplication)

    def _log_function(self, handler):
        """Override Applicaion.log_function so that what to log can be controlled.
        """
        if handler.get_status() < 400:
            log_method = request_log.info
        elif handler.get_status() < 500:
            log_method = request_log.warning
        else:
            log_method = request_log.error
        for i in settings['LOGGING_IGNORE_URLS']:
            if handler.request.uri.startswith(i):
                log_method = request_log.debug
                break

        request_time = 1000.0 * handler.request.request_time()
        log_method("%d %s %.2fms", handler.get_status(),
                   handler._request_summary(), request_time)


_caller_path = None


def _guess_caller():
    """
    try to guess which module import app.py
    """
    import inspect
    global _caller_path

    caller = inspect.stack()[1]
    caller_module = inspect.getmodule(caller[0])
    if hasattr(caller_module, '__file__'):
        _caller_path = os.path.abspath(caller_module.__file__)
    return _caller_path


def guess():
    return _guess_caller()


def _convert_type(raw, before):
    type_ = type(before)
    if type_ is bool:
        if raw == 'True':
            value = True
        elif raw == 'False':
            value = False
        else:
            raise errors.ArgsParseError('Should only be True or False, got: %s' % raw)
    else:
        value = type_(raw)
    return value
